@@ -3,23 +3,24 @@
 # This demonstrates the use of a work queue and worker threads.
 # It really does do more stats/sec when using multiple threads,
 # although the improvement is only about 20-30 percent.
+# (That was 8 years ago.  In 2002, on Linux, I can't measure
+# a speedup. :-( )
 
 # I'm too lazy to write a command line parser for the full find(1)
 # command line syntax, so the predicate it searches for is wired-in,
 # see function selector() below.  (It currently searches for files with
-# group or world write permission.)
+# world write permission.)
 
 # Usage: parfind.py [-w nworkers] [directory] ...
-# Default nworkers is 4, maximum appears to be 8 (on Irix 4.0.2)
+# Default nworkers is 4
 
 
 import sys
 import getopt
-import string
 import time
 import os
 from stat import *
-import thread
+import _thread as thread
 
 
 # Work queue class.  Usage:
@@ -83,7 +84,7 @@ class WorkQ:
             if not job:
                 break
             func, args = job
-            apply(func, args)
+            func(*args)
             self._donework()
 
     def run(self, nworkers):
@@ -98,12 +99,11 @@ class WorkQ:
 # Main program
 
 def main():
-    sys.argv.append("/tmp")
     nworkers = 4
     opts, args = getopt.getopt(sys.argv[1:], '-w:')
     for opt, arg in opts:
         if opt == '-w':
-            nworkers = string.atoi(arg)
+            nworkers = int(arg)
     if not args:
         args = [os.curdir]
 
@@ -115,15 +115,15 @@ def main():
     wq.run(nworkers)
     t2 = time.time()
 
-    sys.stderr.write('Total time ' + `t2-t1` + ' sec.\n')
+    sys.stderr.write('Total time %r sec.\n' % (t2-t1))
 
 
 # The predicate -- defines what files we look for.
 # Feel free to change this to suit your purpose
 
 def selector(dir, name, fullname, stat):
-    # Look for group or world writable files
-    return (stat[ST_MODE] & 0022) != 0
+    # Look for world writable files that are not symlinks
+    return (stat[ST_MODE] & 0o002) != 0 and not S_ISLNK(stat[ST_MODE])
 
 
 # The find procedure -- calls wq.addwork() for subdirectories
@@ -131,19 +131,19 @@ def selector(dir, name, fullname, stat):
 def find(dir, pred, wq):
     try:
         names = os.listdir(dir)
-    except os.error, msg:
-        print `dir`, ':', msg
+    except os.error as msg:
+        print(repr(dir), ':', msg)
         return
     for name in names:
         if name not in (os.curdir, os.pardir):
             fullname = os.path.join(dir, name)
             try:
                 stat = os.lstat(fullname)
-            except os.error, msg:
-                print `fullname`, ':', msg
+            except os.error as msg:
+                print(repr(fullname), ':', msg)
                 continue
             if pred(dir, name, fullname, stat):
-                print fullname
+                print(fullname)
             if S_ISDIR(stat[ST_MODE]):
                 if not os.path.ismount(fullname):
                     wq.addwork(find, (fullname, pred, wq))
