@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/python
 """      turtle-example-suite:
 
              tdemo_tree.py
@@ -41,7 +41,7 @@ def maketree():
     p.setundobuffer(None)
     p.hideturtle()
     p.speed(0)
-    p.tracer(30,0)
+    p.getscreen().tracer(30,0)
     p.left(90)
     p.penup()
     p.forward(-210)
@@ -49,7 +49,7 @@ def maketree():
     t = tree([p], 200, 65, 0.6375)
     for x in t:
         pass
-    print len(p.getscreen().turtles())
+    print(len(p.getscreen().turtles()))
 
 def main():
     a=clock()
@@ -59,5 +59,5 @@ def main():
 
 if __name__ == "__main__":
     msg = main()
-    print msg
+    print(msg)
     mainloop()
