@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/python
 """       turtle-example-suite:
 
             tdemo_yinyang.py
@@ -15,32 +15,32 @@ from turtle import *
 
 def yin(radius, color1, color2):
     width(3)
-    color("black")
-    fill(True)
+    color("black", color1)
+    begin_fill()
     circle(radius/2., 180)
     circle(radius, 180)
     left(180)
     circle(-radius/2., 180)
-    color(color1)
-    fill(True)
-    color(color2)
+    end_fill()
     left(90)
     up()
-    forward(radius*0.375)
+    forward(radius*0.35)
     right(90)
     down()
-    circle(radius*0.125)
+    color(color1, color2)
+    begin_fill()
+    circle(radius*0.15)
+    end_fill()
     left(90)
-    fill(False)
     up()
-    backward(radius*0.375)
+    backward(radius*0.35)
     down()
     left(90)
 
 def main():
     reset()
-    yin(200, "white", "black")
     yin(200, "black", "white")
+    yin(200, "white", "black")
     ht()
     return "Done!"
 
