@@ -3,7 +3,7 @@
 Package containing implementation of all the standard Distutils
 commands."""
 
-__revision__ = "$Id: __init__.py,v 1.14 2000/06/27 01:24:07 gward Exp $"
+__revision__ = "$Id: __init__.py 71500 2009-04-11 20:45:40Z benjamin.peterson $"
 
 __all__ = ['build',
            'build_py',
@@ -17,8 +17,17 @@ __all__ = ['build',
            'install_scripts',
            'install_data',
            'sdist',
+           'register',
            'bdist',
            'bdist_dumb',
            'bdist_rpm',
            'bdist_wininst',
+           'check',
+           'upload',
+           # These two are reserved for future use:
+           #'bdist_sdux',
+           #'bdist_pkgtool',
+           # Note:
+           # bdist_packager is not included because it only provides
+           # an abstract base class
           ]
