@@ -11,9 +11,6 @@ from lib2to3 import fixer_base
 from lib2to3.fixer_util import Call, Name, String, Attr, touch_import
 
 class FixCallable(fixer_base.BaseFix):
-    BM_compatible = True
-
-    order = "pre"
 
     # Ignore callable(*args) or use of keywords.
     # Either could be a hint that the builtin callable() is not being used.
@@ -30,8 +27,8 @@ class FixCallable(fixer_base.BaseFix):
     def transform(self, node, results):
         func = results['func']
 
-        touch_import(None, u'collections', node=node)
+        touch_import(None, 'collections', node=node)
 
-        args = [func.clone(), String(u', ')]
-        args.extend(Attr(Name(u'collections'), Name(u'Callable')))
-        return Call(Name(u'isinstance'), args, prefix=node.prefix)
+        args = [func.clone(), String(', ')]
+        args.extend(Attr(Name('collections'), Name('Callable')))
+        return Call(Name('isinstance'), args, prefix=node.prefix)
