@@ -130,8 +130,10 @@ class CMathTests(unittest.TestCase):
     def test_constants(self):
         e_expected = 2.71828182845904523536
         pi_expected = 3.14159265358979323846
-        self.assertAlmostEqual(cmath.pi, pi_expected)
-        self.assertAlmostEqual(cmath.e,  e_expected)
+        self.assertAlmostEqual(cmath.pi, pi_expected, places=9,
+            msg="cmath.pi is %s; should be %s" % (cmath.pi, pi_expected))
+        self.assertAlmostEqual(cmath.e, e_expected, places=9,
+            msg="cmath.e is %s; should be %s" % (cmath.e, e_expected))
 
     def test_user_object(self):
         # Test automatic calling of __complex__ and __float__ by cmath
