@@ -1,7 +1,9 @@
-from test.test_support import TESTFN, run_unittest
-import mmap
+from test.support import TESTFN, run_unittest, import_module
 import unittest
-import os, re, itertools
+import os, re, itertools, socket
+
+# Skip test if we can't import mmap.
+mmap = import_module('mmap')
 
 PAGESIZE = mmap.PAGESIZE
 
@@ -21,110 +23,107 @@ class MmapTests(unittest.TestCase):
         # Test mmap module on Unix systems and Windows
 
         # Create a file to be mmap'ed.
-        f = open(TESTFN, 'w+')
+        f = open(TESTFN, 'bw+')
         try:
             # Write 2 pages worth of data to the file
-            f.write('\0'* PAGESIZE)
-            f.write('foo')
-            f.write('\0'* (PAGESIZE-3) )
+            f.write(b'\0'* PAGESIZE)
+            f.write(b'foo')
+            f.write(b'\0'* (PAGESIZE-3) )
             f.flush()
             m = mmap.mmap(f.fileno(), 2 * PAGESIZE)
+        finally:
             f.close()
 
-            # Simple sanity checks
+        # Simple sanity checks
 
-            tp = str(type(m))  # SF bug 128713:  segfaulted on Linux
-            self.assertEqual(m.find('foo'), PAGESIZE)
+        tp = str(type(m))  # SF bug 128713:  segfaulted on Linux
+        self.assertEqual(m.find(b'foo'), PAGESIZE)
 
-            self.assertEqual(len(m), 2*PAGESIZE)
+        self.assertEqual(len(m), 2*PAGESIZE)
 
-            self.assertEqual(m[0], '\0')
-            self.assertEqual(m[0:3], '\0\0\0')
+        self.assertEqual(m[0], 0)
+        self.assertEqual(m[0:3], b'\0\0\0')
 
-            # Shouldn't crash on boundary (Issue #5292)
-            self.assertRaises(IndexError, m.__getitem__, len(m))
-            self.assertRaises(IndexError, m.__setitem__, len(m), '\0')
+        # Shouldn't crash on boundary (Issue #5292)
+        self.assertRaises(IndexError, m.__getitem__, len(m))
+        self.assertRaises(IndexError, m.__setitem__, len(m), b'\0')
 
-            # Modify the file's content
-            m[0] = '3'
-            m[PAGESIZE +3: PAGESIZE +3+3] = 'bar'
+        # Modify the file's content
+        m[0] = b'3'[0]
+        m[PAGESIZE +3: PAGESIZE +3+3] = b'bar'
 
-            # Check that the modification worked
-            self.assertEqual(m[0], '3')
-            self.assertEqual(m[0:3], '3\0\0')
-            self.assertEqual(m[PAGESIZE-1 : PAGESIZE + 7], '\0foobar\0')
+        # Check that the modification worked
+        self.assertEqual(m[0], b'3'[0])
+        self.assertEqual(m[0:3], b'3\0\0')
+        self.assertEqual(m[PAGESIZE-1 : PAGESIZE + 7], b'\0foobar\0')
 
-            m.flush()
+        m.flush()
 
-            # Test doing a regular expression match in an mmap'ed file
-            match = re.search('[A-Za-z]+', m)
-            if match is None:
-                self.fail('regex match on mmap failed!')
-            else:
-                start, end = match.span(0)
-                length = end - start
+        # Test doing a regular expression match in an mmap'ed file
+        match = re.search(b'[A-Za-z]+', m)
+        if match is None:
+            self.fail('regex match on mmap failed!')
+        else:
+            start, end = match.span(0)
+            length = end - start
 
-                self.assertEqual(start, PAGESIZE)
-                self.assertEqual(end, PAGESIZE + 6)
+            self.assertEqual(start, PAGESIZE)
+            self.assertEqual(end, PAGESIZE + 6)
 
-            # test seeking around (try to overflow the seek implementation)
-            m.seek(0,0)
-            self.assertEqual(m.tell(), 0)
-            m.seek(42,1)
-            self.assertEqual(m.tell(), 42)
-            m.seek(0,2)
-            self.assertEqual(m.tell(), len(m))
+        # test seeking around (try to overflow the seek implementation)
+        m.seek(0,0)
+        self.assertEqual(m.tell(), 0)
+        m.seek(42,1)
+        self.assertEqual(m.tell(), 42)
+        m.seek(0,2)
+        self.assertEqual(m.tell(), len(m))
 
-            # Try to seek to negative position...
-            self.assertRaises(ValueError, m.seek, -1)
+        # Try to seek to negative position...
+        self.assertRaises(ValueError, m.seek, -1)
 
-            # Try to seek beyond end of mmap...
-            self.assertRaises(ValueError, m.seek, 1, 2)
+        # Try to seek beyond end of mmap...
+        self.assertRaises(ValueError, m.seek, 1, 2)
 
-            # Try to seek to negative position...
-            self.assertRaises(ValueError, m.seek, -len(m)-1, 2)
+        # Try to seek to negative position...
+        self.assertRaises(ValueError, m.seek, -len(m)-1, 2)
 
-            # Try resizing map
+        # Try resizing map
+        try:
+            m.resize(512)
+        except SystemError:
+            # resize() not supported
+            # No messages are printed, since the output of this test suite
+            # would then be different across platforms.
+            pass
+        else:
+            # resize() is supported
+            self.assertEqual(len(m), 512)
+            # Check that we can no longer seek beyond the new size.
+            self.assertRaises(ValueError, m.seek, 513, 0)
+
+            # Check that the underlying file is truncated too
+            # (bug #728515)
+            f = open(TESTFN)
             try:
-                m.resize(512)
-            except SystemError:
-                # resize() not supported
-                # No messages are printed, since the output of this test suite
-                # would then be different across platforms.
-                pass
-            else:
-                # resize() is supported
-                self.assertEqual(len(m), 512)
-                # Check that we can no longer seek beyond the new size.
-                self.assertRaises(ValueError, m.seek, 513, 0)
-
-                # Check that the underlying file is truncated too
-                # (bug #728515)
-                f = open(TESTFN)
                 f.seek(0, 2)
                 self.assertEqual(f.tell(), 512)
+            finally:
                 f.close()
-                self.assertEqual(m.size(), 512)
+            self.assertEqual(m.size(), 512)
 
-            m.close()
-
-        finally:
-            try:
-                f.close()
-            except OSError:
-                pass
+        m.close()
 
     def test_access_parameter(self):
         # Test for "access" keyword parameter
         mapsize = 10
-        open(TESTFN, "wb").write("a"*mapsize)
+        open(TESTFN, "wb").write(b"a"*mapsize)
         f = open(TESTFN, "rb")
         m = mmap.mmap(f.fileno(), mapsize, access=mmap.ACCESS_READ)
-        self.assertEqual(m[:], 'a'*mapsize, "Readonly memory map data incorrect.")
+        self.assertEqual(m[:], b'a'*mapsize, "Readonly memory map data incorrect.")
 
         # Ensuring that readonly mmap can't be slice assigned
         try:
-            m[:] = 'b'*mapsize
+            m[:] = b'b'*mapsize
         except TypeError:
             pass
         else:
@@ -132,7 +131,7 @@ class MmapTests(unittest.TestCase):
 
         # Ensuring that readonly mmap can't be item assigned
         try:
-            m[0] = 'b'
+            m[0] = b'b'
         except TypeError:
             pass
         else:
@@ -141,7 +140,7 @@ class MmapTests(unittest.TestCase):
         # Ensuring that readonly mmap can't be write() to
         try:
             m.seek(0,0)
-            m.write('abc')
+            m.write(b'abc')
         except TypeError:
             pass
         else:
@@ -150,7 +149,7 @@ class MmapTests(unittest.TestCase):
         # Ensuring that readonly mmap can't be write_byte() to
         try:
             m.seek(0,0)
-            m.write_byte('d')
+            m.write_byte(b'd')
         except TypeError:
             pass
         else:
@@ -167,7 +166,7 @@ class MmapTests(unittest.TestCase):
             self.fail("Able to resize readonly memory map")
         f.close()
         del m, f
-        self.assertEqual(open(TESTFN, "rb").read(), 'a'*mapsize,
+        self.assertEqual(open(TESTFN, "rb").read(), b'a'*mapsize,
                "Readonly memory map data file was modified")
 
         # Opening mmap with size too big
@@ -198,8 +197,8 @@ class MmapTests(unittest.TestCase):
         f = open(TESTFN, "r+b")
         m = mmap.mmap(f.fileno(), mapsize, access=mmap.ACCESS_WRITE)
         # Modifying write-through memory map
-        m[:] = 'c'*mapsize
-        self.assertEqual(m[:], 'c'*mapsize,
+        m[:] = b'c'*mapsize
+        self.assertEqual(m[:], b'c'*mapsize,
                "Write-through memory map memory not updated properly.")
         m.flush()
         m.close()
@@ -207,18 +206,18 @@ class MmapTests(unittest.TestCase):
         f = open(TESTFN, 'rb')
         stuff = f.read()
         f.close()
-        self.assertEqual(stuff, 'c'*mapsize,
+        self.assertEqual(stuff, b'c'*mapsize,
                "Write-through memory map data file not updated properly.")
 
         # Opening mmap with access=ACCESS_COPY
         f = open(TESTFN, "r+b")
         m = mmap.mmap(f.fileno(), mapsize, access=mmap.ACCESS_COPY)
         # Modifying copy-on-write memory map
-        m[:] = 'd'*mapsize
-        self.assertEqual(m[:], 'd' * mapsize,
+        m[:] = b'd'*mapsize
+        self.assertEqual(m[:], b'd' * mapsize,
                "Copy-on-write memory map data not written correctly.")
         m.flush()
-        self.assertEqual(open(TESTFN, "rb").read(), 'c'*mapsize,
+        self.assertEqual(open(TESTFN, "rb").read(), b'c'*mapsize,
                "Copy-on-write test data file should not be modified.")
         # Ensuring copy-on-write maps cannot be resized
         self.assertRaises(TypeError, m.resize, 2*mapsize)
@@ -245,9 +244,9 @@ class MmapTests(unittest.TestCase):
     def test_tougher_find(self):
         # Do a tougher .find() test.  SF bug 515943 pointed out that, in 2.2,
         # searching for data with embedded \0 bytes didn't work.
-        f = open(TESTFN, 'w+')
+        f = open(TESTFN, 'wb+')
 
-        data = 'aabaac\x00deef\x00\x00aa\x00'
+        data = b'aabaac\x00deef\x00\x00aa\x00'
         n = len(data)
         f.write(data)
         f.flush()
@@ -258,50 +257,50 @@ class MmapTests(unittest.TestCase):
             for finish in range(start, n+1):
                 slice = data[start : finish]
                 self.assertEqual(m.find(slice), data.find(slice))
-                self.assertEqual(m.find(slice + 'x'), -1)
+                self.assertEqual(m.find(slice + b'x'), -1)
         m.close()
 
     def test_find_end(self):
         # test the new 'end' parameter works as expected
-        f = open(TESTFN, 'w+')
-        data = 'one two ones'
+        f = open(TESTFN, 'wb+')
+        data = b'one two ones'
         n = len(data)
         f.write(data)
         f.flush()
         m = mmap.mmap(f.fileno(), n)
         f.close()
 
-        self.assertEqual(m.find('one'), 0)
-        self.assertEqual(m.find('ones'), 8)
-        self.assertEqual(m.find('one', 0, -1), 0)
-        self.assertEqual(m.find('one', 1), 8)
-        self.assertEqual(m.find('one', 1, -1), 8)
-        self.assertEqual(m.find('one', 1, -2), -1)
+        self.assertEqual(m.find(b'one'), 0)
+        self.assertEqual(m.find(b'ones'), 8)
+        self.assertEqual(m.find(b'one', 0, -1), 0)
+        self.assertEqual(m.find(b'one', 1), 8)
+        self.assertEqual(m.find(b'one', 1, -1), 8)
+        self.assertEqual(m.find(b'one', 1, -2), -1)
 
 
     def test_rfind(self):
         # test the new 'end' parameter works as expected
-        f = open(TESTFN, 'w+')
-        data = 'one two ones'
+        f = open(TESTFN, 'wb+')
+        data = b'one two ones'
         n = len(data)
         f.write(data)
         f.flush()
         m = mmap.mmap(f.fileno(), n)
         f.close()
 
-        self.assertEqual(m.rfind('one'), 8)
-        self.assertEqual(m.rfind('one '), 0)
-        self.assertEqual(m.rfind('one', 0, -1), 8)
-        self.assertEqual(m.rfind('one', 0, -2), 0)
-        self.assertEqual(m.rfind('one', 1, -1), 8)
-        self.assertEqual(m.rfind('one', 1, -2), -1)
+        self.assertEqual(m.rfind(b'one'), 8)
+        self.assertEqual(m.rfind(b'one '), 0)
+        self.assertEqual(m.rfind(b'one', 0, -1), 8)
+        self.assertEqual(m.rfind(b'one', 0, -2), 0)
+        self.assertEqual(m.rfind(b'one', 1, -1), 8)
+        self.assertEqual(m.rfind(b'one', 1, -2), -1)
 
 
     def test_double_close(self):
         # make sure a double close doesn't crash on Solaris (Bug# 665913)
-        f = open(TESTFN, 'w+')
+        f = open(TESTFN, 'wb+')
 
-        f.write(2**16 * 'a') # Arbitrary character
+        f.write(2**16 * b'a') # Arbitrary character
         f.close()
 
         f = open(TESTFN)
@@ -313,33 +312,33 @@ class MmapTests(unittest.TestCase):
     def test_entire_file(self):
         # test mapping of entire file by passing 0 for map length
         if hasattr(os, "stat"):
-            f = open(TESTFN, "w+")
+            f = open(TESTFN, "wb+")
 
-            f.write(2**16 * 'm') # Arbitrary character
+            f.write(2**16 * b'm') # Arbitrary character
             f.close()
 
             f = open(TESTFN, "rb+")
             mf = mmap.mmap(f.fileno(), 0)
             self.assertEqual(len(mf), 2**16, "Map size should equal file size.")
-            self.assertEqual(mf.read(2**16), 2**16 * "m")
+            self.assertEqual(mf.read(2**16), 2**16 * b"m")
             mf.close()
             f.close()
 
     def test_move(self):
         # make move works everywhere (64-bit format problem earlier)
-        f = open(TESTFN, 'w+')
+        f = open(TESTFN, 'wb+')
 
-        f.write("ABCDEabcde") # Arbitrary character
+        f.write(b"ABCDEabcde") # Arbitrary character
         f.flush()
 
         mf = mmap.mmap(f.fileno(), 10)
         mf.move(5, 0, 5)
-        self.assertEqual(mf[:], "ABCDEABCDE", "Map move should have duplicated front 5")
+        self.assertEqual(mf[:], b"ABCDEABCDE", "Map move should have duplicated front 5")
         mf.close()
         f.close()
 
         # more excessive test
-        data = "0123456789"
+        data = b"0123456789"
         for dest in range(len(data)):
             for src in range(len(data)):
                 for count in range(len(data) - max(dest, src)):
@@ -358,28 +357,37 @@ class MmapTests(unittest.TestCase):
                 m.move(source, dest, size)
             except ValueError:
                 pass
-        self.assertRaises(ValueError, m.move, -1, -1, -1)
-        self.assertRaises(ValueError, m.move, -1, -1, 0)
-        self.assertRaises(ValueError, m.move, -1, 0, -1)
-        self.assertRaises(ValueError, m.move, 0, -1, -1)
-        self.assertRaises(ValueError, m.move, -1, 0, 0)
-        self.assertRaises(ValueError, m.move, 0, -1, 0)
-        self.assertRaises(ValueError, m.move, 0, 0, -1)
+
+        offsets = [(-1, -1, -1), (-1, -1, 0), (-1, 0, -1), (0, -1, -1),
+                   (-1, 0, 0), (0, -1, 0), (0, 0, -1)]
+        for source, dest, size in offsets:
+            self.assertRaises(ValueError, m.move, source, dest, size)
+
         m.close()
 
+        m = mmap.mmap(-1, 1) # single byte
+        self.assertRaises(ValueError, m.move, 0, 0, 2)
+        self.assertRaises(ValueError, m.move, 1, 0, 1)
+        self.assertRaises(ValueError, m.move, 0, 1, 1)
+        m.move(0, 0, 1)
+        m.move(0, 0, 0)
+
+
     def test_anonymous(self):
         # anonymous mmap.mmap(-1, PAGE)
         m = mmap.mmap(-1, PAGESIZE)
-        for x in xrange(PAGESIZE):
-            self.assertEqual(m[x], '\0', "anonymously mmap'ed contents should be zero")
+        for x in range(PAGESIZE):
+            self.assertEqual(m[x], 0,
+                             "anonymously mmap'ed contents should be zero")
 
-        for x in xrange(PAGESIZE):
-            m[x] = ch = chr(x & 255)
-            self.assertEqual(m[x], ch)
+        for x in range(PAGESIZE):
+            b = x & 0xff
+            m[x] = b
+            self.assertEqual(m[x], b)
 
     def test_extended_getslice(self):
         # Test extended slicing by comparing with list slicing.
-        s = "".join(chr(c) for c in reversed(range(256)))
+        s = bytes(reversed(range(256)))
         m = mmap.mmap(-1, len(s))
         m[:] = s
         self.assertEqual(m[:], s)
@@ -393,7 +401,7 @@ class MmapTests(unittest.TestCase):
 
     def test_extended_set_del_slice(self):
         # Test extended slicing by comparing with list slicing.
-        s = "".join(chr(c) for c in reversed(range(256)))
+        s = bytes(reversed(range(256)))
         m = mmap.mmap(-1, len(s))
         indices = (0, None, 1, 3, 19, 300, -1, -2, -31, -300)
         for start in indices:
@@ -406,16 +414,16 @@ class MmapTests(unittest.TestCase):
                     # Make sure we have a slice of exactly the right length,
                     # but with different data.
                     data = L[start:stop:step]
-                    data = "".join(reversed(data))
+                    data = bytes(reversed(data))
                     L[start:stop:step] = data
                     m[start:stop:step] = data
-                    self.assertEquals(m[:], "".join(L))
+                    self.assertEquals(m[:], bytes(L))
 
     def make_mmap_file (self, f, halfsize):
         # Write 2 pages worth of data to the file
-        f.write ('\0' * halfsize)
-        f.write ('foo')
-        f.write ('\0' * (halfsize - 3))
+        f.write (b'\0' * halfsize)
+        f.write (b'foo')
+        f.write (b'\0' * (halfsize - 3))
         f.flush ()
         return mmap.mmap (f.fileno(), 0)
 
@@ -444,7 +452,7 @@ class MmapTests(unittest.TestCase):
             # Try valid offset, hopefully 8192 works on all OSes
             f = open(TESTFN, "r+b")
             m = mmap.mmap(f.fileno(), mapsize - halfsize, offset=halfsize)
-            self.assertEqual(m[0:3], 'foo')
+            self.assertEqual(m[0:3], b'foo')
             f.close()
 
             # Try resizing map
@@ -458,7 +466,7 @@ class MmapTests(unittest.TestCase):
                 # Check that we can no longer seek beyond the new size.
                 self.assertRaises(ValueError, m.seek, 513, 0)
                 # Check that the content is not changed
-                self.assertEqual(m[0:3], 'foo')
+                self.assertEqual(m[0:3], b'foo')
 
                 # Check that the underlying file is truncated too
                 f = open(TESTFN)
@@ -486,52 +494,52 @@ class MmapTests(unittest.TestCase):
         if not hasattr(mmap, 'PROT_READ'):
             return
         mapsize = 10
-        open(TESTFN, "wb").write("a"*mapsize)
+        open(TESTFN, "wb").write(b"a"*mapsize)
         f = open(TESTFN, "rb")
         m = mmap.mmap(f.fileno(), mapsize, prot=mmap.PROT_READ)
         self.assertRaises(TypeError, m.write, "foo")
         f.close()
 
     def test_error(self):
-        self.assert_(issubclass(mmap.error, EnvironmentError))
-        self.assert_("mmap.error" in str(mmap.error))
+        self.assertTrue(issubclass(mmap.error, EnvironmentError))
+        self.assertTrue("mmap.error" in str(mmap.error))
 
     def test_io_methods(self):
-        data = "0123456789"
-        open(TESTFN, "wb").write("x"*len(data))
+        data = b"0123456789"
+        open(TESTFN, "wb").write(b"x"*len(data))
         f = open(TESTFN, "r+b")
         m = mmap.mmap(f.fileno(), len(data))
         f.close()
         # Test write_byte()
-        for i in xrange(len(data)):
+        for i in range(len(data)):
             self.assertEquals(m.tell(), i)
-            m.write_byte(data[i:i+1])
+            m.write_byte(data[i])
             self.assertEquals(m.tell(), i+1)
-        self.assertRaises(ValueError, m.write_byte, "x")
+        self.assertRaises(ValueError, m.write_byte, b"x"[0])
         self.assertEquals(m[:], data)
         # Test read_byte()
         m.seek(0)
-        for i in xrange(len(data)):
+        for i in range(len(data)):
             self.assertEquals(m.tell(), i)
-            self.assertEquals(m.read_byte(), data[i:i+1])
+            self.assertEquals(m.read_byte(), data[i])
             self.assertEquals(m.tell(), i+1)
         self.assertRaises(ValueError, m.read_byte)
         # Test read()
         m.seek(3)
-        self.assertEquals(m.read(3), "345")
+        self.assertEquals(m.read(3), b"345")
         self.assertEquals(m.tell(), 6)
         # Test write()
         m.seek(3)
-        m.write("bar")
+        m.write(b"bar")
         self.assertEquals(m.tell(), 6)
-        self.assertEquals(m[:], "012bar6789")
+        self.assertEquals(m[:], b"012bar6789")
         m.seek(8)
-        self.assertRaises(ValueError, m.write, "bar")
+        self.assertRaises(ValueError, m.write, b"bar")
 
     if os.name == 'nt':
         def test_tagname(self):
-            data1 = "0123456789"
-            data2 = "abcdefghij"
+            data1 = b"0123456789"
+            data2 = b"abcdefghij"
             assert len(data1) == len(data2)
 
             # Test same tag
@@ -564,7 +572,7 @@ class MmapTests(unittest.TestCase):
             m.close()
 
             # Should not crash (Issue 5385)
-            open(TESTFN, "wb").write("x"*10)
+            open(TESTFN, "wb").write(b"x"*10)
             f = open(TESTFN, "r+b")
             m = mmap.mmap(f.fileno(), 0)
             f.close()
@@ -578,6 +586,16 @@ class MmapTests(unittest.TestCase):
                 pass
             m.close()
 
+        def test_invalid_descriptor(self):
+            # socket file descriptors are valid, but out of range
+            # for _get_osfhandle, causing a crash when validating the
+            # parameters to _get_osfhandle.
+            s = socket.socket()
+            try:
+                with self.assertRaises(mmap.error):
+                    m = mmap.mmap(s.fileno(), 10)
+            finally:
+                s.close()
 
 def test_main():
     run_unittest(MmapTests)
