@@ -6,8 +6,9 @@
 
 import unittest
 import threading
-import Queue
+import queue as pyqueue
 import time
+import io
 import sys
 import os
 import gc
@@ -17,23 +18,19 @@ import copy
 import socket
 import random
 import logging
-from StringIO import StringIO
-from test import test_support
+import test.support
 
 
-# Work around broken sem_open implementations
-try:
-    import multiprocessing.synchronize
-except ImportError, e:
-    from test.test_support import TestSkipped
-    raise TestSkipped(e)
+# Skip tests if _multiprocessing wasn't built.
+_multiprocessing = test.support.import_module('_multiprocessing')
+# Skip tests if sem_open implementation is broken.
+test.support.import_module('multiprocessing.synchronize')
 
 import multiprocessing.dummy
 import multiprocessing.connection
 import multiprocessing.managers
 import multiprocessing.heap
 import multiprocessing.pool
-import _multiprocessing
 
 from multiprocessing import util
 
@@ -41,7 +38,8 @@ from multiprocessing import util
 #
 #
 
-latin = str
+def latin(s):
+    return s.encode('latin')
 
 #
 # Constants
@@ -364,22 +362,22 @@ class _TestQueue(BaseTestCase):
         put = TimingWrapper(queue.put)
         put_nowait = TimingWrapper(queue.put_nowait)
 
-        self.assertRaises(Queue.Full, put, 7, False)
+        self.assertRaises(pyqueue.Full, put, 7, False)
         self.assertTimingAlmostEqual(put.elapsed, 0)
 
-        self.assertRaises(Queue.Full, put, 7, False, None)
+        self.assertRaises(pyqueue.Full, put, 7, False, None)
         self.assertTimingAlmostEqual(put.elapsed, 0)
 
-        self.assertRaises(Queue.Full, put_nowait, 7)
+        self.assertRaises(pyqueue.Full, put_nowait, 7)
         self.assertTimingAlmostEqual(put_nowait.elapsed, 0)
 
-        self.assertRaises(Queue.Full, put, 7, True, TIMEOUT1)
+        self.assertRaises(pyqueue.Full, put, 7, True, TIMEOUT1)
         self.assertTimingAlmostEqual(put.elapsed, TIMEOUT1)
 
-        self.assertRaises(Queue.Full, put, 7, False, TIMEOUT2)
+        self.assertRaises(pyqueue.Full, put, 7, False, TIMEOUT2)
         self.assertTimingAlmostEqual(put.elapsed, 0)
 
-        self.assertRaises(Queue.Full, put, 7, True, timeout=TIMEOUT3)
+        self.assertRaises(pyqueue.Full, put, 7, True, timeout=TIMEOUT3)
         self.assertTimingAlmostEqual(put.elapsed, TIMEOUT3)
 
         child_can_start.set()
@@ -431,22 +429,22 @@ class _TestQueue(BaseTestCase):
         get = TimingWrapper(queue.get)
         get_nowait = TimingWrapper(queue.get_nowait)
 
-        self.assertRaises(Queue.Empty, get, False)
+        self.assertRaises(pyqueue.Empty, get, False)
         self.assertTimingAlmostEqual(get.elapsed, 0)
 
-        self.assertRaises(Queue.Empty, get, False, None)
+        self.assertRaises(pyqueue.Empty, get, False, None)
         self.assertTimingAlmostEqual(get.elapsed, 0)
 
-        self.assertRaises(Queue.Empty, get_nowait)
+        self.assertRaises(pyqueue.Empty, get_nowait)
         self.assertTimingAlmostEqual(get_nowait.elapsed, 0)
 
-        self.assertRaises(Queue.Empty, get, True, TIMEOUT1)
+        self.assertRaises(pyqueue.Empty, get, True, TIMEOUT1)
         self.assertTimingAlmostEqual(get.elapsed, TIMEOUT1)
 
-        self.assertRaises(Queue.Empty, get, False, TIMEOUT2)
+        self.assertRaises(pyqueue.Empty, get, False, TIMEOUT2)
         self.assertTimingAlmostEqual(get.elapsed, 0)
 
-        self.assertRaises(Queue.Empty, get, timeout=TIMEOUT3)
+        self.assertRaises(pyqueue.Empty, get, timeout=TIMEOUT3)
         self.assertTimingAlmostEqual(get.elapsed, TIMEOUT3)
 
         proc.join()
@@ -480,7 +478,7 @@ class _TestQueue(BaseTestCase):
         # check that all expected items are in the queue
         for i in range(20):
             self.assertEqual(queue.get(), i)
-        self.assertRaises(Queue.Empty, queue.get, False)
+        self.assertRaises(pyqueue.Empty, queue.get, False)
 
         p.join()
 
@@ -511,12 +509,12 @@ class _TestQueue(BaseTestCase):
             return
 
         workers = [self.Process(target=self._test_task_done, args=(queue,))
-                   for i in xrange(4)]
+                   for i in range(4)]
 
         for p in workers:
             p.start()
 
-        for i in xrange(10):
+        for i in range(10):
             queue.put(i)
 
         queue.join()
@@ -690,11 +688,11 @@ class _TestCondition(BaseTestCase):
             t.start()
 
         # wait for them all to sleep
-        for i in xrange(6):
+        for i in range(6):
             sleeping.acquire()
 
         # check they have all timed out
-        for i in xrange(6):
+        for i in range(6):
             woken.acquire()
         self.assertReturnsIfImplemented(0, get_value, woken)
 
@@ -712,7 +710,7 @@ class _TestCondition(BaseTestCase):
             t.start()
 
         # wait for them to all sleep
-        for i in xrange(6):
+        for i in range(6):
             sleeping.acquire()
 
         # check no process/thread has woken up
@@ -753,20 +751,22 @@ class _TestEvent(BaseTestCase):
 
         # Removed temporaily, due to API shear, this does not
         # work with threading._Event objects. is_set == isSet
-        #self.assertEqual(event.is_set(), False)
+        self.assertEqual(event.is_set(), False)
 
-        self.assertEqual(wait(0.0), None)
+        # Removed, threading.Event.wait() will return the value of the __flag
+        # instead of None. API Shear with the semaphore backed mp.Event
+        self.assertEqual(wait(0.0), False)
         self.assertTimingAlmostEqual(wait.elapsed, 0.0)
-        self.assertEqual(wait(TIMEOUT1), None)
+        self.assertEqual(wait(TIMEOUT1), False)
         self.assertTimingAlmostEqual(wait.elapsed, TIMEOUT1)
 
         event.set()
 
         # See note above on the API differences
-        # self.assertEqual(event.is_set(), True)
-        self.assertEqual(wait(), None)
+        self.assertEqual(event.is_set(), True)
+        self.assertEqual(wait(), True)
         self.assertTimingAlmostEqual(wait.elapsed, 0.0)
-        self.assertEqual(wait(TIMEOUT1), None)
+        self.assertEqual(wait(TIMEOUT1), True)
         self.assertTimingAlmostEqual(wait.elapsed, 0.0)
         # self.assertEqual(event.is_set(), True)
 
@@ -775,7 +775,7 @@ class _TestEvent(BaseTestCase):
         #self.assertEqual(event.is_set(), False)
 
         self.Process(target=self._test_event, args=(event,)).start()
-        self.assertEqual(wait(), None)
+        self.assertEqual(wait(), True)
 
 #
 #
@@ -887,16 +887,16 @@ class _TestArray(BaseTestCase):
         if self.TYPE != 'processes':
             return
 
-        arr1 = self.Array('i', range(10))
+        arr1 = self.Array('i', list(range(10)))
         lock1 = arr1.get_lock()
         obj1 = arr1.get_obj()
 
-        arr2 = self.Array('i', range(10), lock=None)
+        arr2 = self.Array('i', list(range(10)), lock=None)
         lock2 = arr2.get_lock()
         obj2 = arr2.get_obj()
 
         lock = self.Lock()
-        arr3 = self.Array('i', range(10), lock=lock)
+        arr3 = self.Array('i', list(range(10)), lock=lock)
         lock3 = arr3.get_lock()
         obj3 = arr3.get_obj()
         self.assertEqual(lock, lock3)
@@ -920,14 +920,14 @@ class _TestContainers(BaseTestCase):
     ALLOWED_TYPES = ('manager',)
 
     def test_list(self):
-        a = self.list(range(10))
-        self.assertEqual(a[:], range(10))
+        a = self.list(list(range(10)))
+        self.assertEqual(a[:], list(range(10)))
 
         b = self.list()
         self.assertEqual(b[:], [])
 
-        b.extend(range(5))
-        self.assertEqual(b[:], range(5))
+        b.extend(list(range(5)))
+        self.assertEqual(b[:], list(range(5)))
 
         self.assertEqual(b[2], 2)
         self.assertEqual(b[2:10], [2,3,4])
@@ -937,7 +937,7 @@ class _TestContainers(BaseTestCase):
 
         self.assertEqual(b + [5, 6], [0, 1, 2, 3, 4, 0, 1, 2, 3, 4, 5, 6])
 
-        self.assertEqual(a[:], range(10))
+        self.assertEqual(a[:], list(range(10)))
 
         d = [a, b]
         e = self.list(d)
@@ -952,7 +952,7 @@ class _TestContainers(BaseTestCase):
 
     def test_dict(self):
         d = self.dict()
-        indices = range(65, 70)
+        indices = list(range(65, 70))
         for i in indices:
             d[i] = chr(i)
         self.assertEqual(d.copy(), dict((i, chr(i)) for i in indices))
@@ -987,9 +987,15 @@ class _TestPool(BaseTestCase):
 
     def test_map(self):
         pmap = self.pool.map
-        self.assertEqual(pmap(sqr, range(10)), map(sqr, range(10)))
-        self.assertEqual(pmap(sqr, range(100), chunksize=20),
-                         map(sqr, range(100)))
+        self.assertEqual(pmap(sqr, list(range(10))), list(map(sqr, list(range(10)))))
+        self.assertEqual(pmap(sqr, list(range(100)), chunksize=20),
+                         list(map(sqr, list(range(100)))))
+
+    def test_map_chunksize(self):
+        try:
+            self.pool.map_async(sqr, [], chunksize=1).get(timeout=TIMEOUT1)
+        except multiprocessing.TimeoutError:
+            self.fail("pool.map_async with chunksize stalled on null list")
 
     def test_async(self):
         res = self.pool.apply_async(sqr, (7, TIMEOUT1,))
@@ -1004,25 +1010,25 @@ class _TestPool(BaseTestCase):
         self.assertTimingAlmostEqual(get.elapsed, TIMEOUT2)
 
     def test_imap(self):
-        it = self.pool.imap(sqr, range(10))
-        self.assertEqual(list(it), map(sqr, range(10)))
+        it = self.pool.imap(sqr, list(range(10)))
+        self.assertEqual(list(it), list(map(sqr, list(range(10)))))
 
-        it = self.pool.imap(sqr, range(10))
+        it = self.pool.imap(sqr, list(range(10)))
         for i in range(10):
-            self.assertEqual(it.next(), i*i)
-        self.assertRaises(StopIteration, it.next)
+            self.assertEqual(next(it), i*i)
+        self.assertRaises(StopIteration, it.__next__)
 
-        it = self.pool.imap(sqr, range(1000), chunksize=100)
+        it = self.pool.imap(sqr, list(range(1000)), chunksize=100)
         for i in range(1000):
-            self.assertEqual(it.next(), i*i)
-        self.assertRaises(StopIteration, it.next)
+            self.assertEqual(next(it), i*i)
+        self.assertRaises(StopIteration, it.__next__)
 
     def test_imap_unordered(self):
-        it = self.pool.imap_unordered(sqr, range(1000))
-        self.assertEqual(sorted(it), map(sqr, range(1000)))
+        it = self.pool.imap_unordered(sqr, list(range(1000)))
+        self.assertEqual(sorted(it), list(map(sqr, list(range(1000)))))
 
-        it = self.pool.imap_unordered(sqr, range(1000), chunksize=53)
-        self.assertEqual(sorted(it), map(sqr, range(1000)))
+        it = self.pool.imap_unordered(sqr, list(range(1000)), chunksize=53)
+        self.assertEqual(sorted(it), list(map(sqr, list(range(1000)))))
 
     def test_make_pool(self):
         p = multiprocessing.Pool(3)
@@ -1063,8 +1069,10 @@ class _TestZZZNumberOfObjects(BaseTestCase):
         multiprocessing.active_children()  # discard dead process objs
         gc.collect()                       # do garbage collection
         refs = self.manager._number_of_objects()
+        debug_info = self.manager._debug_info()
         if refs != EXPECTED_NUMBER:
-            print self.manager._debug_info()
+            print(self.manager._debug_info())
+            print(debug_info)
 
         self.assertEqual(refs, EXPECTED_NUMBER)
 
@@ -1083,14 +1091,14 @@ class FooBar(object):
         return '_h()'
 
 def baz():
-    for i in xrange(10):
+    for i in range(10):
         yield i*i
 
 class IteratorProxy(BaseProxy):
     _exposed_ = ('next', '__next__')
     def __iter__(self):
         return self
-    def next(self):
+    def __next__(self):
         return self._callmethod('next')
     def __next__(self):
         return self._callmethod('__next__')
@@ -1139,7 +1147,7 @@ class _TestMyManager(BaseTestCase):
 # Test of connecting to a remote server and using xmlrpclib for serialization
 #
 
-_queue = Queue.Queue()
+_queue = pyqueue.Queue()
 def get_queue():
     return _queue
 
@@ -1205,9 +1213,9 @@ class _TestManagerRestart(BaseTestCase):
 
     def test_rapid_restart(self):
         authkey = os.urandom(32)
+        port = test.support.find_unused_port()
         manager = QueueManager(
-            address=('localhost', 0), authkey=authkey, serializer=SERIALIZER)
-        addr = manager.get_server().address
+            address=('localhost', port), authkey=authkey, serializer=SERIALIZER)
         manager.start()
 
         p = self.Process(target=self._putter, args=(manager.address, authkey))
@@ -1217,7 +1225,7 @@ class _TestManagerRestart(BaseTestCase):
         del queue
         manager.shutdown()
         manager = QueueManager(
-            address=addr, authkey=authkey, serializer=SERIALIZER)
+            address=('localhost', port), authkey=authkey, serializer=SERIALIZER)
         manager.start()
         manager.shutdown()
 
@@ -1246,7 +1254,7 @@ class _TestConnection(BaseTestCase):
         seq = [1, 2.25, None]
         msg = latin('hello world')
         longmsg = msg * 10
-        arr = array.array('i', range(4))
+        arr = array.array('i', list(range(4)))
 
         if self.TYPE == 'processes':
             self.assertEqual(type(conn.fileno()), int)
@@ -1276,7 +1284,7 @@ class _TestConnection(BaseTestCase):
             self.assertEqual(conn.send_bytes(longmsg), None)
             try:
                 res = conn.recv_bytes_into(buffer)
-            except multiprocessing.BufferTooShort, e:
+            except multiprocessing.BufferTooShort as e:
                 self.assertEqual(e.args, (longmsg,))
             else:
                 self.fail('expected BufferTooShort, got %s' % res)
@@ -1495,7 +1503,7 @@ class _TestHeap(BaseTestCase):
         blocks = []
 
         # create and destroy lots of blocks of different sizes
-        for i in xrange(iterations):
+        for i in range(iterations):
             size = int(random.lognormvariate(0, 1) * 1000)
             b = multiprocessing.heap.BufferWrapper(size)
             blocks.append(b)
@@ -1509,7 +1517,7 @@ class _TestHeap(BaseTestCase):
         # verify the state of the heap
         all = []
         occupied = 0
-        for L in heap._len_to_seq.values():
+        for L in list(heap._len_to_seq.values()):
             for arena, start, stop in L:
                 all.append((heap._arenas.index(arena), start, stop,
                             stop-start, 'free'))
@@ -1560,10 +1568,10 @@ class _TestSharedCTypes(BaseTestCase):
             return
 
         x = Value('i', 7, lock=lock)
-        y = Value(c_double, 1.0/3.0, lock=lock)
+        y = Value(ctypes.c_double, 1.0/3.0, lock=lock)
         foo = Value(_Foo, 3, 2, lock=lock)
-        arr = self.Array('d', range(10), lock=lock)
-        string = self.Array('c', 20, lock=lock)
+        arr = Array('d', list(range(10)), lock=lock)
+        string = Array('c', 20, lock=lock)
         string.value = 'hello'
 
         p = self.Process(target=self._double, args=(x, y, foo, arr, string))
@@ -1746,7 +1754,7 @@ def create_test_cases(Mixin, type):
     glob = globals()
     Type = type[0].upper() + type[1:]
 
-    for name in glob.keys():
+    for name in list(glob.keys()):
         if name.startswith('_Test'):
             base = glob[name]
             if type in base.ALLOWED_TYPES:
@@ -1832,6 +1840,36 @@ class OtherTest(unittest.TestCase):
                           multiprocessing.connection.answer_challenge,
                           _FakeConnection(), b'abc')
 
+#
+# Test Manager.start()/Pool.__init__() initializer feature - see issue 5585
+#
+
+def initializer(ns):
+    ns.test += 1
+
+class TestInitializers(unittest.TestCase):
+    def setUp(self):
+        self.mgr = multiprocessing.Manager()
+        self.ns = self.mgr.Namespace()
+        self.ns.test = 0
+
+    def tearDown(self):
+        self.mgr.shutdown()
+
+    def test_manager_initializer(self):
+        m = multiprocessing.managers.SyncManager()
+        self.assertRaises(TypeError, m.start, 1)
+        m.start(initializer, (self.ns,))
+        self.assertEqual(self.ns.test, 1)
+        m.shutdown()
+
+    def test_pool_initializer(self):
+        self.assertRaises(TypeError, multiprocessing.Pool, initializer=1)
+        p = multiprocessing.Pool(1, initializer, (self.ns,))
+        p.close()
+        p.join()
+        self.assertEqual(self.ns.test, 1)
+
 #
 # Issue 5155, 5313, 5331: Test process in processes
 # Verifies os.close(sys.stdin.fileno) vs. sys.stdin.close() behavior
@@ -1840,7 +1878,7 @@ class OtherTest(unittest.TestCase):
 def _ThisSubProcess(q):
     try:
         item = q.get(block=False)
-    except Queue.Empty:
+    except pyqueue.Empty:
         pass
 
 def _TestProcess(q):
@@ -1891,14 +1929,15 @@ class TestStdinBadfiledescriptor(unittest.TestCase):
         p.join()
 
     def test_flushing(self):
-        sio = StringIO()
+        sio = io.StringIO()
         flike = _file_like(sio)
         flike.write('foo')
         proc = multiprocessing.Process(target=lambda: flike.flush())
         flike.flush()
         assert sio.getvalue() == 'foo'
 
-testcases_other = [OtherTest, TestInvalidHandle, TestStdinBadfiledescriptor]
+testcases_other = [OtherTest, TestInvalidHandle, TestInitializers,
+                   TestStdinBadfiledescriptor]
 
 #
 #
@@ -1909,11 +1948,10 @@ def test_main(run=None):
         try:
             lock = multiprocessing.RLock()
         except OSError:
-            from test.test_support import TestSkipped
-            raise TestSkipped("OSError raises on RLock creation, see issue 3111!")
+            raise unittest.SkipTest("OSError raises on RLock creation, see issue 3111!")
 
     if run is None:
-        from test.test_support import run_unittest as run
+        from test.support import run_unittest as run
 
     util.get_temp_dir()     # creates temp directory for use by all processes
 
@@ -1934,9 +1972,7 @@ def test_main(run=None):
 
     loadTestsFromTestCase = unittest.defaultTestLoader.loadTestsFromTestCase
     suite = unittest.TestSuite(loadTestsFromTestCase(tc) for tc in testcases)
-    with test_support._check_py3k_warnings(
-            (".+__(get|set)slice__ has been removed", DeprecationWarning)):
-        run(suite)
+    run(suite)
 
     ThreadsMixin.pool.terminate()
     ProcessesMixin.pool.terminate()
