@@ -291,7 +291,7 @@ class GeneralModuleTests(unittest.TestCase):
                 # On some versions, this loses a reference
                 orig = sys.getrefcount(__name__)
                 socket.getnameinfo(__name__,0)
-            except SystemError:
+            except TypeError:
                 if sys.getrefcount(__name__) != orig:
                     self.fail("socket.getnameinfo loses a reference")
 
