@@ -31,7 +31,7 @@
  *   PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* CVS: $Id: _cursesmodule.c 74397 2009-08-13 09:15:16Z georg.brandl $ */
+/* CVS: $Id: _cursesmodule.c 78323 2010-02-22 16:59:30Z andrew.kuchling $ */
 
 /*
 
@@ -890,14 +890,17 @@ PyCursesWindow_GetKey(PyCursesWindowObject *self, PyObject *args)
     /* getch() returns ERR in nodelay mode */
     PyErr_SetString(PyCursesError, "no input");
     return NULL;
-  } else if (rtn<=255)
+  } else if (rtn<=255) {
     return Py_BuildValue("C", rtn);
-  else
+  } else {
+    const char *knp;
 #if defined(__NetBSD__)
-    return PyUnicode_FromString(unctrl(rtn));
+    knp = unctrl(rtn);
 #else
-    return PyUnicode_FromString((const char *)keyname(rtn));
+    knp = keyname(rtn);
 #endif
+    return PyUnicode_FromString((knp == NULL) ? "" : knp);
+  }
 }
 
 static PyObject *
@@ -2656,8 +2659,6 @@ PyCurses_Use_Env(PyObject *self, PyObject *args)
 {
   int flag;
 
-  PyCursesInitialised
-
   switch(PyTuple_Size(args)) {
   case 1:
     if (!PyArg_ParseTuple(args,"i;True(1), False(0)",&flag))
