@@ -75,7 +75,7 @@ Py_ssize_t
 _PyObject_LengthHint(PyObject *o, Py_ssize_t defaultvalue)
 {
 	static PyObject *hintstrobj = NULL;
-	PyObject *ro;
+	PyObject *ro, *hintmeth;
 	Py_ssize_t rv;
 
 	/* try o.__len__() */
@@ -83,25 +83,24 @@ _PyObject_LengthHint(PyObject *o, Py_ssize_t defaultvalue)
 	if (rv >= 0)
 		return rv;
 	if (PyErr_Occurred()) {
-		if (!PyErr_ExceptionMatches(PyExc_TypeError) &&
-			!PyErr_ExceptionMatches(PyExc_AttributeError))
-				return -1;
+		if (!PyErr_ExceptionMatches(PyExc_TypeError))
+			return -1;
 		PyErr_Clear();
 	}
 
-	/* cache a hashed version of the attribute string */
-	if (hintstrobj == NULL) {
-		hintstrobj = PyUnicode_InternFromString("__length_hint__");
-		if (hintstrobj == NULL)
-			return -1;
-	}
-
 	/* try o.__length_hint__() */
-	ro = PyObject_CallMethodObjArgs(o, hintstrobj, NULL);
+        hintmeth = _PyObject_LookupSpecial(o, "__length_hint__", &hintstrobj);
+	if (hintmeth == NULL) {
+		if (PyErr_Occurred())
+			return -1;
+		else
+			return defaultvalue;
+	}
+	ro = PyObject_CallFunctionObjArgs(hintmeth, NULL);
+	Py_DECREF(hintmeth);
 	if (ro == NULL) {
-		if (!PyErr_ExceptionMatches(PyExc_TypeError) &&
-			!PyErr_ExceptionMatches(PyExc_AttributeError))
-				return -1;
+		if (!PyErr_ExceptionMatches(PyExc_TypeError))
+			return -1;
 		PyErr_Clear();
 		return defaultvalue;
 	}
@@ -135,7 +134,7 @@ PyObject_GetItem(PyObject *o, PyObject *key)
 					  "be integer, not '%.200s'", key);
 	}
 
-	return type_error("'%.200s' object is unsubscriptable", o);
+	return type_error("'%.200s' object is not subscriptable", o);
 }
 
 int
@@ -332,7 +331,7 @@ PyObject_GetBuffer(PyObject *obj, Py_buffer *view, int flags)
 {
 	if (!PyObject_CheckBuffer(obj)) {
 		PyErr_Format(PyExc_TypeError,
-                             "'%100s' does not have the buffer interface",
+                             "'%100s' does not support the buffer interface",
                              Py_TYPE(obj)->tp_name);
 		return -1;
 	}
@@ -1872,7 +1871,7 @@ PySequence_Fast(PyObject *v, const char *m)
 
 /* Iterate over seq.  Result depends on the operation:
    PY_ITERSEARCH_COUNT:	 -1 if error, else # of times obj appears in seq.
-   PY_ITERSEARCH_INDEX:	 0-based index of first occurence of obj in seq;
+   PY_ITERSEARCH_INDEX:	 0-based index of first occurrence of obj in seq;
 	set ValueError and return -1 if none found; also return -1 on error.
    Py_ITERSEARCH_CONTAINS:  return 1 if obj in seq, else 0; -1 on error.
 */
@@ -2579,14 +2578,8 @@ PyObject_IsInstance(PyObject *inst, PyObject *cls)
 		Py_LeaveRecursiveCall();
 		return r;
 	}
-	if (name == NULL) {
-		name = PyUnicode_InternFromString("__instancecheck__");
-		if (name == NULL)
-			return -1;
-	}
-	checker = PyObject_GetAttr(cls, name);
-	if (checker == NULL && PyErr_Occurred())
-		PyErr_Clear();
+
+	checker = _PyObject_LookupSpecial(cls, "__instancecheck__", &name);
 	if (checker != NULL) {
 		PyObject *res;
 		int ok = -1;
@@ -2603,6 +2596,8 @@ PyObject_IsInstance(PyObject *inst, PyObject *cls)
 		}
 		return ok;
 	}
+	else if (PyErr_Occurred())
+		return -1;
 	return recursive_isinstance(inst, cls);
 }
 
@@ -2628,9 +2623,8 @@ int
 PyObject_IsSubclass(PyObject *derived, PyObject *cls)
 {
 	static PyObject *name = NULL;
-	PyObject *t, *v, *tb;
 	PyObject *checker;
-	
+
 	if (PyTuple_Check(cls)) {
 		Py_ssize_t i;
 		Py_ssize_t n;
@@ -2649,14 +2643,8 @@ PyObject_IsSubclass(PyObject *derived, PyObject *cls)
 		Py_LeaveRecursiveCall();
 		return r;
 	}
-	if (name == NULL) {
-		name = PyUnicode_InternFromString("__subclasscheck__");
-		if (name == NULL)
-			return -1;
-	}
-	PyErr_Fetch(&t, &v, &tb);
-	checker = PyObject_GetAttr(cls, name);
-	PyErr_Restore(t, v, tb);
+
+	checker = _PyObject_LookupSpecial(cls, "__subclasscheck__", &name);
 	if (checker != NULL) {
 		PyObject *res;
 		int ok = -1;
@@ -2673,6 +2661,8 @@ PyObject_IsSubclass(PyObject *derived, PyObject *cls)
 		}
 		return ok;
 	}
+	else if (PyErr_Occurred())
+		return -1;
 	return recursive_issubclass(derived, cls);
 }
 
@@ -2725,7 +2715,6 @@ PyObject *
 PyIter_Next(PyObject *iter)
 {
 	PyObject *result;
-	assert(PyIter_Check(iter));
 	result = (*iter->ob_type->tp_iternext)(iter);
 	if (result == NULL &&
 	    PyErr_Occurred() &&
