@@ -2100,12 +2100,12 @@ dict_iter(PyDictObject *dict)
 }
 
 PyDoc_STRVAR(dictionary_doc,
-"dict() -> new empty dictionary.\n"
+"dict() -> new empty dictionary\n"
 "dict(mapping) -> new dictionary initialized from a mapping object's\n"
-"    (key, value) pairs.\n"
-"dict(seq) -> new dictionary initialized as if via:\n"
+"    (key, value) pairs\n"
+"dict(iterable) -> new dictionary initialized as if via:\n"
 "    d = {}\n"
-"    for k, v in seq:\n"
+"    for k, v in iterable:\n"
 "        d[k] = v\n"
 "dict(**kwargs) -> new dictionary initialized with the name=value pairs\n"
 "    in the keyword argument list.  For example:  dict(one=1, two=2)");
