@@ -2,6 +2,7 @@
 /* Check for interrupts */
 
 #include "Python.h"
+#include "pythread.h"
 
 #ifdef QUICKWIN
 
@@ -92,14 +93,6 @@ PyOS_InterruptOccurred(void)
 #endif /* MSDOS && !QUICKWIN */
 
 
-#ifdef macintosh
-
-/* The Mac interrupt code has moved to macglue.c */
-#define OK
-
-#endif /* macintosh */
-
-
 #ifndef OK
 
 /* Default version -- for real operating systems and for Standard C */
@@ -134,18 +127,14 @@ intcatcher(int sig)
 	case 0:
 		break;
 	case 1:
-#ifdef RISCOS
-		fprintf(stderr, message);
-#else
 		write(2, message, strlen(message));
-#endif
 		break;
 	case 2:
 		interrupted = 0;
 		Py_Exit(1);
 		break;
 	}
-	signal(SIGINT, intcatcher);
+	PyOS_setsig(SIGINT, intcatcher);
 	Py_AddPendingCall(checksignals_witharg, NULL);
 }
 
@@ -154,23 +143,14 @@ static void (*old_siginthandler)(int) = SIG_DFL;
 void
 PyOS_InitInterrupts(void)
 {
-	if ((old_siginthandler = signal(SIGINT, SIG_IGN)) != SIG_IGN)
-		signal(SIGINT, intcatcher);
-#ifdef HAVE_SIGINTERRUPT
-	/* This is for SunOS and other modern BSD derivatives.
-	   It means that system calls (like read()) are not restarted
-	   after an interrupt.  This is necessary so interrupting a
-	   read() or readline() call works as expected.
-	   XXX On old BSD (pure 4.2 or older) you may have to do this
-	   differently! */
-	siginterrupt(SIGINT, 1);
-#endif /* HAVE_SIGINTERRUPT */
+	if ((old_siginthandler = PyOS_setsig(SIGINT, SIG_IGN)) != SIG_IGN)
+		PyOS_setsig(SIGINT, intcatcher);
 }
 
 void
 PyOS_FiniInterrupts(void)
 {
-	signal(SIGINT, old_siginthandler);
+	PyOS_setsig(SIGINT, old_siginthandler);
 }
 
 int
@@ -189,5 +169,6 @@ PyOS_AfterFork(void)
 {
 #ifdef WITH_THREAD
 	PyEval_ReInitThreads();
+	PyThread_ReInitTLS();
 #endif
 }
