@@ -13,6 +13,7 @@
    - check for duplicate definitions of names (instead of fatal err)
 */
 
+#include "Python.h"
 #include "pgenheaders.h"
 #include "grammar.h"
 #include "node.h"
@@ -21,13 +22,10 @@
 
 int Py_DebugFlag;
 int Py_VerboseFlag;
+int Py_IgnoreEnvironmentFlag;
 
 /* Forward */
 grammar *getgrammar(char *filename);
-#ifdef THINK_C
-int main(int, char **);
-char *askfile(void);
-#endif
 
 void
 Py_Exit(int sts)
@@ -42,11 +40,6 @@ main(int argc, char **argv)
 	FILE *fp;
 	char *filename, *graminit_h, *graminit_c;
 	
-#ifdef THINK_C
-	filename = askfile();
-	graminit_h = askfile();
-	graminit_c = askfile();
-#else
 	if (argc != 4) {
 		fprintf(stderr,
 			"usage: %s grammar graminit.h graminit.c\n", argv[0]);
@@ -55,14 +48,14 @@ main(int argc, char **argv)
 	filename = argv[1];
 	graminit_h = argv[2];
 	graminit_c = argv[3];
-#endif
 	g = getgrammar(filename);
 	fp = fopen(graminit_c, "w");
 	if (fp == NULL) {
 		perror(graminit_c);
 		Py_Exit(1);
 	}
-	printf("Writing %s ...\n", graminit_c);
+	if (Py_DebugFlag)
+		printf("Writing %s ...\n", graminit_c);
 	printgrammar(g, fp);
 	fclose(fp);
 	fp = fopen(graminit_h, "w");
@@ -70,7 +63,8 @@ main(int argc, char **argv)
 		perror(graminit_h);
 		Py_Exit(1);
 	}
-	printf("Writing %s ...\n", graminit_h);
+	if (Py_DebugFlag)
+		printf("Writing %s ...\n", graminit_h);
 	printnonterminals(g, fp);
 	fclose(fp);
 	Py_Exit(0);
@@ -110,7 +104,7 @@ getgrammar(char *filename)
 					putc(' ', stderr);
 			}
 			fprintf(stderr, "^\n");
-			PyMem_DEL(err.text);
+			PyObject_FREE(err.text);
 		}
 		Py_Exit(1);
 	}
@@ -122,54 +116,32 @@ getgrammar(char *filename)
 	return g;
 }
 
-#ifdef THINK_C
-char *
-askfile(void)
+/* Can't happen in pgen */
+PyObject*
+PyErr_Occurred()
 {
-	char buf[256];
-	static char name[256];
-	printf("Input file name: ");
-	if (fgets(buf, sizeof buf, stdin) == NULL) {
-		printf("EOF\n");
-		Py_Exit(1);
-	}
-	/* XXX The (unsigned char *) case is needed by THINK C 3.0 */
-	if (sscanf(/*(unsigned char *)*/buf, " %s ", name) != 1) {
-		printf("No file\n");
-		Py_Exit(1);
-	}
-	return name;
+	return 0;
 }
-#endif
 
 void
-Py_FatalError(char *msg)
+Py_FatalError(const char *msg)
 {
 	fprintf(stderr, "pgen: FATAL ERROR: %s\n", msg);
 	Py_Exit(1);
 }
 
-#ifdef macintosh
-/* ARGSUSED */
-int
-guesstabsize(char *path)
-{
-	return 4;
-}
-#endif
-
 /* No-nonsense my_readline() for tokenizer.c */
 
 char *
-PyOS_Readline(char *prompt)
+PyOS_Readline(FILE *sys_stdin, FILE *sys_stdout, char *prompt)
 {
 	size_t n = 1000;
-	char *p = PyMem_MALLOC(n);
+	char *p = (char *)PyMem_MALLOC(n);
 	char *q;
 	if (p == NULL)
 		return NULL;
 	fprintf(stderr, "%s", prompt);
-	q = fgets(p, n, stdin);
+	q = fgets(p, n, sys_stdin);
 	if (q == NULL) {
 		*p = '\0';
 		return p;
@@ -177,9 +149,17 @@ PyOS_Readline(char *prompt)
 	n = strlen(p);
 	if (n > 0 && p[n-1] != '\n')
 		p[n-1] = '\n';
-	return PyMem_REALLOC(p, n+1);
+	return (char *)PyMem_REALLOC(p, n+1);
 }
 
+/* No-nonsense fgets */
+char *
+Py_UniversalNewlineFgets(char *buf, int n, FILE *stream, PyObject *fobj)
+{
+	return fgets(buf, n, stream);
+}
+
+
 #include <stdarg.h>
 
 void
