@@ -18,6 +18,10 @@
 
 #ifdef HAVE_DLFCN_H
 #include <dlfcn.h>
+#else
+#if defined(PYOS_OS2) && defined(PYCC_GCC)
+#include "dlfcn.h"
+#endif
 #endif
 
 #if (defined(__OpenBSD__) || defined(__NetBSD__)) && !defined(__ELF__)
@@ -31,16 +35,32 @@ const struct filedescr _PyImport_DynLoadFiletab[] = {
 #ifdef __CYGWIN__
 	{".dll", "rb", C_EXTENSION},
 	{"module.dll", "rb", C_EXTENSION},
+#else
+#if defined(PYOS_OS2) && defined(PYCC_GCC)
+	{".pyd", "rb", C_EXTENSION},
+	{".dll", "rb", C_EXTENSION},
+#else
+#ifdef __VMS
+        {".exe", "rb", C_EXTENSION},
+        {".EXE", "rb", C_EXTENSION},
+        {"module.exe", "rb", C_EXTENSION},
+        {"MODULE.EXE", "rb", C_EXTENSION},
 #else
 	{".so", "rb", C_EXTENSION},
 	{"module.so", "rb", C_EXTENSION},
+#endif
+#endif
 #endif
 	{0, 0}
 };
 
 static struct {
 	dev_t dev;
+#ifdef __VMS
+	ino_t ino[3];
+#else
 	ino_t ino;
+#endif
 	void *handle;
 } handles[128];
 static int nhandles = 0;
@@ -62,7 +82,7 @@ dl_funcptr _PyImport_GetDynLoadFunc(const char *fqname, const char *shortname,
 	}
 
 	PyOS_snprintf(funcname, sizeof(funcname), 
-		      LEAD_UNDERSCORE "init%.200s", shortname);
+		      LEAD_UNDERSCORE "PyInit_%.200s", shortname);
 
 	if (fp != NULL) {
 		int i;
@@ -78,19 +98,42 @@ dl_funcptr _PyImport_GetDynLoadFunc(const char *fqname, const char *shortname,
 		}
 		if (nhandles < 128) {
 			handles[nhandles].dev = statb.st_dev;
+#ifdef __VMS
+			handles[nhandles].ino[0] = statb.st_ino[0];
+			handles[nhandles].ino[1] = statb.st_ino[1];
+			handles[nhandles].ino[2] = statb.st_ino[2];
+#else
 			handles[nhandles].ino = statb.st_ino;
+#endif
 		}
 	}
 
-        dlopenflags = PyThreadState_Get()->interp->dlopenflags;
+#if !(defined(PYOS_OS2) && defined(PYCC_GCC))
+        dlopenflags = PyThreadState_GET()->interp->dlopenflags;
+#endif
 
 	if (Py_VerboseFlag)
-		printf("dlopen(\"%s\", %x);\n", pathname, dlopenflags);
+		PySys_WriteStderr("dlopen(\"%s\", %x);\n", pathname, 
+				  dlopenflags);
+
+#ifdef __VMS
+	/* VMS currently don't allow a pathname, use a logical name instead */
+	/* Concatenate 'python_module_' and shortname */
+	/* so "import vms.bar" will use the logical python_module_bar */
+	/* As C module use only one name space this is probably not a */
+	/* important limitation */
+	PyOS_snprintf(pathbuf, sizeof(pathbuf), "python_module_%-.200s", 
+		      shortname);
+	pathname = pathbuf;
+#endif
 
 	handle = dlopen(pathname, dlopenflags);
 
 	if (handle == NULL) {
-		PyErr_SetString(PyExc_ImportError, dlerror());
+		const char *error = dlerror();
+		if (error == NULL)
+			error = "unknown dlopen() error";
+		PyErr_SetString(PyExc_ImportError, error);
 		return NULL;
 	}
 	if (fp != NULL && nhandles < 128)
