@@ -1,27 +1,30 @@
 
 /* Support for dynamic loading of extension modules */
 
-#include <windows.h>
+#include "Python.h"
+
+#ifdef HAVE_DIRECT_H
 #include <direct.h>
+#endif
 #include <ctype.h>
 
-#include "Python.h"
 #include "importdl.h"
+#include <windows.h>
+
+// "activation context" magic - see dl_nt.c...
+extern ULONG_PTR _Py_ActivateActCtx();
+void _Py_DeactivateActCtx(ULONG_PTR cookie);
 
 const struct filedescr _PyImport_DynLoadFiletab[] = {
 #ifdef _DEBUG
 	{"_d.pyd", "rb", C_EXTENSION},
-	{"_d.dll", "rb", C_EXTENSION},
 #else
 	{".pyd", "rb", C_EXTENSION},
-	{".dll", "rb", C_EXTENSION},
 #endif
 	{0, 0}
 };
 
 
-#ifdef MS_WIN32
-
 /* Case insensitive string compare, to avoid any dependencies on particular
    C RTL implementations */
 
@@ -118,6 +121,10 @@ static char *GetPythonImport (HINSTANCE hModule)
 	   string constant holding the import name is located. */
 
 	if (DWORD_AT(dllbase + opt_offset + num_dict_off) >= 2) {
+		/* We have at least 2 tables - the import table is the second
+		   one.  But still it may be that the table size is zero */
+		if (0 == DWORD_AT(dllbase + opt_offset + import_off + sizeof(DWORD)))
+			return NULL;
 		import_data = dllbase + DWORD_AT(dllbase +
 						 opt_offset +
 						 import_off);
@@ -130,7 +137,11 @@ static char *GetPythonImport (HINSTANCE hModule)
 				/* Ensure python prefix is followed only
 				   by numbers to the end of the basename */
 				pch = import_name + 6;
+#ifdef _DEBUG
+				while (*pch && pch[0] != '_' && pch[1] != 'd' && pch[2] != '.') {
+#else
 				while (*pch && *pch != '.') {
+#endif
 					if (*pch >= '0' && *pch <= '9') {
 						pch++;
 					} else {
@@ -150,7 +161,6 @@ static char *GetPythonImport (HINSTANCE hModule)
 
 	return NULL;
 }
-#endif /* MS_WIN32 */
 
 
 dl_funcptr _PyImport_GetDynLoadFunc(const char *fqname, const char *shortname,
@@ -159,53 +169,67 @@ dl_funcptr _PyImport_GetDynLoadFunc(const char *fqname, const char *shortname,
 	dl_funcptr p;
 	char funcname[258], *import_python;
 
-	sprintf(funcname, "init%.200s", shortname);
+	PyOS_snprintf(funcname, sizeof(funcname), "PyInit_%.200s", shortname);
 
-#ifdef MS_WIN32
 	{
 		HINSTANCE hDLL = NULL;
 		char pathbuf[260];
 		LPTSTR dummy;
+		unsigned int old_mode;
+		ULONG_PTR cookie = 0;
 		/* We use LoadLibraryEx so Windows looks for dependent DLLs 
 		    in directory of pathname first.  However, Windows95
 		    can sometimes not work correctly unless the absolute
 		    path is used.  If GetFullPathName() fails, the LoadLibrary
 		    will certainly fail too, so use its error code */
+
+		/* Don't display a message box when Python can't load a DLL */
+		old_mode = SetErrorMode(SEM_FAILCRITICALERRORS);
+
 		if (GetFullPathName(pathname,
 				    sizeof(pathbuf),
 				    pathbuf,
-				    &dummy))
+				    &dummy)) {
+			ULONG_PTR cookie = _Py_ActivateActCtx();
 			/* XXX This call doesn't exist in Windows CE */
 			hDLL = LoadLibraryEx(pathname, NULL,
 					     LOAD_WITH_ALTERED_SEARCH_PATH);
+			_Py_DeactivateActCtx(cookie);
+		}
+
+		/* restore old error mode settings */
+		SetErrorMode(old_mode);
+
 		if (hDLL==NULL){
-			char errBuf[256];
+			PyObject *message;
 			unsigned int errorCode;
 
 			/* Get an error string from Win32 error code */
-			char theInfo[256]; /* Pointer to error text
+			wchar_t theInfo[256]; /* Pointer to error text
 					      from system */
 			int theLength; /* Length of error text */
 
 			errorCode = GetLastError();
 
-			theLength = FormatMessage(
-				FORMAT_MESSAGE_FROM_SYSTEM, /* flags */
+			theLength = FormatMessageW(
+				FORMAT_MESSAGE_FROM_SYSTEM |
+				FORMAT_MESSAGE_IGNORE_INSERTS, /* flags */
 				NULL, /* message source */
 				errorCode, /* the message (error) ID */
-				0, /* default language environment */
-				(LPTSTR) theInfo, /* the buffer */
+				MAKELANGID(LANG_NEUTRAL,
+					   SUBLANG_DEFAULT),
+				           /* Default language */
+				theInfo, /* the buffer */
 				sizeof(theInfo), /* the buffer size */
 				NULL); /* no additional format args. */
 
 			/* Problem: could not get the error message.
 			   This should not happen if called correctly. */
 			if (theLength == 0) {
-				sprintf(errBuf,
+				message = PyUnicode_FromFormat(
 					"DLL load failed with error code %d",
 					errorCode);
 			} else {
-				size_t len;
 				/* For some reason a \r\n
 				   is appended to the text */
 				if (theLength >= 2 &&
@@ -214,27 +238,34 @@ dl_funcptr _PyImport_GetDynLoadFunc(const char *fqname, const char *shortname,
 					theLength -= 2;
 					theInfo[theLength] = '\0';
 				}
-				strcpy(errBuf, "DLL load failed: ");
-				len = strlen(errBuf);
-				strncpy(errBuf+len, theInfo,
-					sizeof(errBuf)-len);
-				errBuf[sizeof(errBuf)-1] = '\0';
+				message = PyUnicode_FromString(
+					"DLL load failed: ");
+
+				PyUnicode_AppendAndDel(&message, 
+					PyUnicode_FromUnicode(
+						theInfo, 
+						theLength));
 			}
-			PyErr_SetString(PyExc_ImportError, errBuf);
-		return NULL;
+			PyErr_SetObject(PyExc_ImportError, message);
+			Py_XDECREF(message);
+			return NULL;
 		} else {
 			char buffer[256];
 
-			sprintf(buffer,"python%d%d.dll",
-				PY_MAJOR_VERSION,PY_MINOR_VERSION);
+#ifdef _DEBUG
+			PyOS_snprintf(buffer, sizeof(buffer), "python%d%d_d.dll",
+#else
+			PyOS_snprintf(buffer, sizeof(buffer), "python%d%d.dll",
+#endif
+				      PY_MAJOR_VERSION,PY_MINOR_VERSION);
 			import_python = GetPythonImport(hDLL);
 
 			if (import_python &&
 			    strcasecmp(buffer,import_python)) {
-				sprintf(buffer,
-					"Module use of %s conflicts "
-					"with this version of Python.",
-					import_python);
+				PyOS_snprintf(buffer, sizeof(buffer),
+					      "Module use of %.150s conflicts "
+					      "with this version of Python.",
+					      import_python);
 				PyErr_SetString(PyExc_ImportError,buffer);
 				FreeLibrary(hDLL);
 				return NULL;
@@ -242,29 +273,6 @@ dl_funcptr _PyImport_GetDynLoadFunc(const char *fqname, const char *shortname,
 		}
 		p = GetProcAddress(hDLL, funcname);
 	}
-#endif /* MS_WIN32 */
-#ifdef MS_WIN16
-	{
-		HINSTANCE hDLL;
-		char pathbuf[16];
-		if (strchr(pathname, '\\') == NULL &&
-		    strchr(pathname, '/') == NULL)
-		{
-			/* Prefix bare filename with ".\" */
-			sprintf(pathbuf, ".\\%-.13s", pathname);
-			pathname = pathbuf;
-		}
-		hDLL = LoadLibrary(pathname);
-		if (hDLL < HINSTANCE_ERROR){
-			char errBuf[256];
-			sprintf(errBuf,
-				"DLL load failed with error code %d", hDLL);
-			PyErr_SetString(PyExc_ImportError, errBuf);
-			return NULL;
-		}
-		p = GetProcAddress(hDLL, funcname);
-	}
-#endif /* MS_WIN16 */
 
 	return p;
 }
