@@ -107,7 +107,7 @@ sys_displayhook(PyObject *self, PyObject *o)
 PyDoc_STRVAR(displayhook_doc,
 "displayhook(object) -> None\n"
 "\n"
-"Print an object to sys.stdout and also save it in builtins.\n"
+"Print an object to sys.stdout and also save it in builtins._\n"
 );
 
 static PyObject *
@@ -1066,7 +1066,7 @@ settrace() -- set the global debug tracing function\n\
 
 /* Subversion branch and revision management */
 static const char _patchlevel_revision[] = PY_PATCHLEVEL_REVISION;
-static const char headurl[] = "$HeadURL: http://svn.python.org/projects/python/tags/r311/Python/sysmodule.c $";
+static const char headurl[] = "$HeadURL: http://svn.python.org/projects/python/tags/r312/Python/sysmodule.c $";
 static int svn_initialized;
 static char patchlevel_revision[50]; /* Just the number */
 static char branch[50];
