@@ -1,19 +1,18 @@
 # Python MSI Generator
 # (C) 2003 Martin v. Loewis
 # See "FOO" in comments refers to MSDN sections with the title FOO.
-import msilib, schema, sequence, os, glob, time, re, shutil, zipfile
+import msilib, schema, sequence, os, glob, time, re, shutil
 from msilib import Feature, CAB, Directory, Dialog, Binary, add_data
 import uisample
 from win32com.client import constants
 from distutils.spawn import find_executable
 from uuids import product_codes
-import tempfile
 
 # Settings can be overridden in config.py below
 # 0 for official python.org releases
 # 1 for intermediate releases by anybody, with
 # a new product code for every package.
-snapshot = int(os.environ.get("SNAPSHOT", "1"))
+snapshot = 1
 # 1 means that file extension is px, not py,
 # and binaries start with x
 testpackage = 0
@@ -22,17 +21,13 @@ srcdir = os.path.abspath("../..")
 # Text to be displayed as the version in dialogs etc.
 # goes into file name and ProductCode. Defaults to
 # current_version.day for Snapshot, current_version otherwise
-full_current_version = os.environ.get("CURRENT_VERSION")
+full_current_version = None
 # Is Tcl available at all?
 have_tcl = True
 # path to PCbuild directory
-PCBUILD=os.environ.get("PCBUILD", "PCbuild")
+PCBUILD="PCbuild"
 # msvcrt version
 MSVCR = "90"
-# Name of certificate in default store to sign MSI with
-certname = os.environ.get("CERTNAME", None)
-# Make a zip file containing the PDB files for this build?
-pdbzip = True
 
 try:
     from config import *
@@ -91,7 +86,6 @@ extensions = [
     'unicodedata.pyd',
     'winsound.pyd',
     '_elementtree.pyd',
-    '_bsddb.pyd',
     '_socket.pyd',
     '_ssl.pyd',
     '_testcapi.pyd',
@@ -117,6 +111,8 @@ pythondll_uuid = {
     "25":"{2e41b118-38bd-4c1b-a840-6977efd1b911}",
     "26":"{34ebecac-f046-4e1c-b0e3-9bac3cdaacfa}",
     "27":"{4fe21c76-1760-437b-a2f2-99909130a175}",
+    "30":"{6953bc3b-6768-4291-8410-7914ce6e2ca8}",
+    "31":"{4afcba0b-13e4-47c3-bebe-477428b46913}",
     } [major+minor]
 
 # Compute the name that Sphinx gives to the docfile
@@ -132,54 +128,55 @@ docfile = 'python%s%s%s.chm' % (major, minor, docfile)
 
 # Build the mingw import library, libpythonXY.a
 # This requires 'nm' and 'dlltool' executables on your PATH
-def build_mingw_lib(dll_path, def_file, dll_file, mingw_lib):
+def build_mingw_lib(lib_file, def_file, dll_file, mingw_lib):
     warning = "WARNING: %s - libpythonXX.a not built"
-    gendef = find_executable('gendef')
+    nm = find_executable('nm')
     dlltool = find_executable('dlltool')
 
-    if not gendef or not dlltool:
-        print warning % "gendef and/or dlltool were not found"
+    if not nm or not dlltool:
+        print(warning % "nm and/or dlltool were not found")
         return False
 
-    gendef_command = '%s - %s' % (gendef, dll_path)
+    nm_command = '%s -Cs %s' % (nm, lib_file)
     dlltool_command = "%s --dllname %s --def %s --output-lib %s" % \
         (dlltool, dll_file, def_file, mingw_lib)
-    if msilib.Win64:
-        dlltool_command += " -m i386:x86-64"
-    else:
-        dlltool_command += " -m i386 --as-flags=--32"
+    export_match = re.compile(r"^_imp__(.*) in python\d+\.dll").match
 
     f = open(def_file,'w')
-    gendef_pipe = os.popen(gendef_command)
-    for line in gendef_pipe.readlines():
-        print >>f, line
+    f.write("LIBRARY %s\n" % dll_file)
+    f.write("EXPORTS\n")
+
+    nm_pipe = os.popen(nm_command)
+    for line in nm_pipe.readlines():
+        m = export_match(line)
+        if m:
+            f.write(m.group(1)+"\n")
     f.close()
-    exit = gendef_pipe.close()
+    exit = nm_pipe.close()
 
     if exit:
-        print warning % "gendef did not run successfully"
+        print(warning % "nm did not run successfully")
         return False
 
     if os.system(dlltool_command) != 0:
-        print warning % "dlltool did not run successfully"
+        print(warning % "dlltool did not run successfully")
         return False
 
     return True
 
 # Target files (.def and .a) go in PCBuild directory
-dll_path = os.path.join(srcdir, PCBUILD, "python%s%s.dll" % (major, minor))
+lib_file = os.path.join(srcdir, PCBUILD, "python%s%s.lib" % (major, minor))
 def_file = os.path.join(srcdir, PCBUILD, "python%s%s.def" % (major, minor))
 dll_file = "python%s%s.dll" % (major, minor)
 mingw_lib = os.path.join(srcdir, PCBUILD, "libpython%s%s.a" % (major, minor))
 
-# Determine the target architecture
-if os.system("nmake /nologo /c /f msisupport.mak") != 0:
-    raise RuntimeError("'nmake /f msisupport.mak' failed")
+have_mingw = build_mingw_lib(lib_file, def_file, dll_file, mingw_lib)
+
+# Determine the target architechture
 dll_path = os.path.join(srcdir, PCBUILD, dll_file)
 msilib.set_arch_from_file(dll_path)
 if msilib.pe_type(dll_path) != msilib.pe_type("msisupport.dll"):
-    raise SystemError, "msisupport.dll for incorrect architecture"
-
+    raise SystemError("msisupport.dll for incorrect architecture")
 if msilib.Win64:
     upgrade_code = upgrade_code_64
     # Bump the last digit of the code by one, so that 32-bit and 64-bit
@@ -187,8 +184,6 @@ if msilib.Win64:
     digit = hex((int(product_code[-2],16)+1)%16)[-1]
     product_code = product_code[:-2] + digit + '}'
 
-have_mingw = build_mingw_lib(dll_path, def_file, dll_file, mingw_lib)
-
 if testpackage:
     ext = 'px'
     testprefix = 'x'
@@ -224,8 +219,7 @@ def build_database():
     # schema represents the installer 2.0 database schema.
     # sequence is the set of standard sequences
     # (ui/execute, admin/advt/install)
-    msiname = "python-%s%s.msi" % (full_current_version, msilib.arch_ext)
-    db = msilib.init_database(msiname,
+    db = msilib.init_database("python-%s%s.msi" % (full_current_version, msilib.arch_ext),
                   schema, ProductName="Python "+full_current_version+productsuffix,
                   ProductCode=product_code,
                   ProductVersion=current_version,
@@ -248,7 +242,7 @@ def build_database():
                               ("ProductLine", "Python%s%s" % (major, minor)),
                              ])
     db.Commit()
-    return db, msiname
+    return db
 
 def remove_old_versions(db):
     "Fill the upgrade table."
@@ -362,7 +356,7 @@ def add_ui(db):
 
     # Bitmaps
     if not os.path.exists(srcdir+r"\PC\python_icon.exe"):
-        raise "Run icons.mak in PC directory"
+        raise RuntimeError("Run icons.mak in PC directory")
     add_data(db, "Binary",
              [("PythonWin", msilib.Binary(r"%s\PCbuild\installer.bmp" % srcdir)), # 152x328 pixels
               ("py.ico",msilib.Binary(srcdir+r"\PC\py.ico")),
@@ -375,6 +369,8 @@ def add_ui(db):
     # UpdateEditIDLE sets the REGISTRY.tcl component into
     # the installed/uninstalled state according to both the
     # Extensions and TclTk features.
+    if os.system("nmake /nologo /c /f msisupport.mak") != 0:
+        raise RuntimeError("'nmake /f msisupport.mak' failed")
     add_data(db, "Binary", [("Script", msilib.Binary("msisupport.dll"))])
     # See "Custom Action Type 1"
     if msilib.Win64:
@@ -407,10 +403,8 @@ def add_ui(db):
               ("VerdanaRed9", "Verdana", 9, 255, 0),
              ])
 
-    compileargs = r'-Wi "[TARGETDIR]Lib\compileall.py" -f -x "bad_coding|badsyntax|site-packages|py3_" "[TARGETDIR]Lib"'
+    compileargs = r'-Wi "[TARGETDIR]Lib\compileall.py" -f -x bad_coding|badsyntax|site-packages|py2_ "[TARGETDIR]Lib"'
     lib2to3args = r'-c "import lib2to3.pygram, lib2to3.patcomp;lib2to3.patcomp.PatternCompiler()"'
-    updatepipargs = r'-m ensurepip -U --default-pip'
-    removepipargs = r'-B -m ensurepip._uninstall'
     # See "CustomAction Table"
     add_data(db, "CustomAction", [
         # msidbCustomActionTypeFirstSequence + msidbCustomActionTypeTextData + msidbCustomActionTypeProperty
@@ -422,13 +416,9 @@ def add_ui(db):
         ("SetDLLDirToSystem32", 307, "DLLDIR", SystemFolderName),
         # msidbCustomActionTypeExe + msidbCustomActionTypeSourceFile
         # See "Custom Action Type 18"
-        # msidbCustomActionTypeInScript (1024); run during actual installation
-        # msidbCustomActionTypeNoImpersonate (2048); run action in system account, not user account
-        ("CompilePyc", 18+1024+2048, "python.exe", compileargs),
-        ("CompilePyo", 18+1024+2048, "python.exe", "-O "+compileargs),
-        ("CompileGrammar", 18+1024+2048, "python.exe", lib2to3args),
-        ("UpdatePip", 18+1024+2048, "python.exe", updatepipargs),
-        ("RemovePip", 18+1024+2048, "python.exe", removepipargs),
+        ("CompilePyc", 18, "python.exe", compileargs),
+        ("CompilePyo", 18, "python.exe", "-O "+compileargs),
+        ("CompileGrammar", 18, "python.exe", lib2to3args),
         ])
 
     # UI Sequences, see "InstallUISequence Table", "Using a Sequence Table"
@@ -450,29 +440,22 @@ def add_ui(db):
               ("SetDLLDirToTarget", 'DLLDIR=""', 751),
              ])
 
-    # Prepend TARGETDIR to the system path, and remove it on uninstall.
-    add_data(db, "Environment",
-             [("PathAddition", "=-*Path", "[TARGETDIR];[TARGETDIR]Scripts;[~]", "REGISTRY.path")])
-
     # Execute Sequences
     add_data(db, "InstallExecuteSequence",
             [("InitialTargetDir", 'TARGETDIR=""', 750),
              ("SetDLLDirToSystem32", 'DLLDIR="" and ' + sys32cond, 751),
              ("SetDLLDirToTarget", 'DLLDIR="" and not ' + sys32cond, 752),
              ("UpdateEditIDLE", None, 1050),
-             # run command if install state of pip changes to INSTALLSTATE_LOCAL
-             # run after InstallFiles
-             ("UpdatePip", "&pip_feature=3", 4001),
-             # remove pip when state changes to INSTALLSTATE_ABSENT
-             # run before RemoveFiles
-             ("RemovePip", "&pip_feature=2", 3499),
-             ("CompilePyc", "COMPILEALL", 4002),
-             ("CompilePyo", "COMPILEALL", 4003),
-             ("CompileGrammar", "COMPILEALL", 4004),
+             ("CompilePyc", "COMPILEALL", 6800),
+             ("CompilePyo", "COMPILEALL", 6801),
+             ("CompileGrammar", "COMPILEALL", 6802),
             ])
     add_data(db, "AdminExecuteSequence",
             [("InitialTargetDir", 'TARGETDIR=""', 750),
              ("SetDLLDirToTarget", 'DLLDIR=""', 751),
+             ("CompilePyc", "COMPILEALL", 6800),
+             ("CompilePyo", "COMPILEALL", 6801),
+             ("CompileGrammar", "COMPILEALL", 6802),
             ])
 
     #####################################################################
@@ -507,7 +490,7 @@ def add_ui(db):
 
     exit_dialog = PyDialog(db, "ExitDialog", x, y, w, h, modal, title,
                          "Finish", "Finish", "Finish")
-    exit_dialog.title("Complete the [ProductName] Installer")
+    exit_dialog.title("Completing the [ProductName] Installer")
     exit_dialog.back("< Back", "Finish", active = 0)
     exit_dialog.cancel("Cancel", "Back", active = 0)
     exit_dialog.text("Acknowledgements", 135, 95, 220, 120, 0x30003,
@@ -517,9 +500,9 @@ def add_ui(db):
       "    would still be Python for DOS.")
 
     c = exit_dialog.text("warning", 135, 200, 220, 40, 0x30003,
-            "{\\VerdanaRed9}Warning: Python 3.3.0 is the last "
-            "Python release for Windows 2000.")
-    c.condition("Hide", "VersionNT > 500")
+            "{\\VerdanaRed9}Warning: Python 2.5.x is the last "
+            "Python release for Windows 9x.")
+    c.condition("Hide", "NOT Version9X")
 
     exit_dialog.text("Description", 135, 235, 220, 20, 0x30003,
                "Click the Finish button to exit the Installer.")
@@ -680,11 +663,11 @@ def add_ui(db):
     c=features.xbutton("Advanced", "Advanced", None, 0.30)
     c.event("SpawnDialog", "AdvancedDlg")
 
-    c=features.text("ItemDescription", 140, 180, 210, 40, 3,
+    c=features.text("ItemDescription", 140, 180, 210, 30, 3,
                   "Multiline description of the currently selected item.")
     c.mapping("SelectionDescription","Text")
 
-    c=features.text("ItemSize", 140, 225, 210, 33, 3,
+    c=features.text("ItemSize", 140, 210, 210, 45, 3,
                     "The size of the currently selected item.")
     c.mapping("SelectionSize", "Text")
 
@@ -838,8 +821,7 @@ def add_features(db):
     # (i.e. additional Python libraries) need to follow the parent feature.
     # Features that have no advertisement trigger (e.g. the test suite)
     # must not support advertisement
-    global default_feature, tcltk, htmlfiles, tools, testsuite
-    global ext_feature, private_crt, prepend_path, pip_feature
+    global default_feature, tcltk, htmlfiles, tools, testsuite, ext_feature, private_crt
     default_feature = Feature(db, "DefaultFeature", "Python",
                               "Python Interpreter and Libraries",
                               1, directory = "TARGETDIR")
@@ -861,22 +843,9 @@ def add_features(db):
     tools = Feature(db, "Tools", "Utility Scripts",
                     "Python utility scripts (Tools/", 9,
                     parent = default_feature, attributes=2)
-    pip_feature = Feature(db, "pip_feature", "pip",
-                    "Install or upgrade pip, a tool for installing and managing "
-                    "Python packages.", 11,
-                    parent = default_feature, attributes=2|8)
     testsuite = Feature(db, "Testsuite", "Test suite",
-                        "Python test suite (Lib/test/)", 13,
+                        "Python test suite (Lib/test/)", 11,
                         parent = default_feature, attributes=2|8)
-    # prepend_path is an additional feature which is to be off by default.
-    # Since the default level for the above features is 1, this needs to be
-    # at least level higher.
-    prepend_path = Feature(db, "PrependPath", "Add python.exe to Path",
-                           "Prepend [TARGETDIR] to the system Path variable. "
-                           "This allows you to type 'python' into a command "
-                           "prompt without needing the full path.", 15,
-                           parent = default_feature, attributes=2|8,
-                           level=2)
 
 def extract_msvcr90():
     # Find the redistributable files
@@ -907,13 +876,13 @@ def generate_license():
     for name, pat, file in (("bzip2","bzip2-*", "LICENSE"),
                       ("Berkeley DB", "db-*", "LICENSE"),
                       ("openssl", "openssl-*", "LICENSE"),
-                      ("Tcl", "tcl-8*", "license.terms"),
-                      ("Tk", "tk-8*", "license.terms"),
+                      ("Tcl", "tcl8*", "license.terms"),
+                      ("Tk", "tk8*", "license.terms"),
                       ("Tix", "tix-*", "license.terms")):
         out.write("\nThis copy of Python includes a copy of %s, which is licensed under the following terms:\n\n" % name)
-        dirs = glob.glob(srcdir+"/externals/"+pat)
+        dirs = glob.glob(srcdir+"/../"+pat)
         if not dirs:
-            raise ValueError, "Could not find "+srcdir+"/externals/"+pat
+            raise ValueError, "Could not find "+srcdir+"/../"+pat
         if len(dirs) > 2:
             raise ValueError, "Multiple copies of "+pat
         dir = dirs[0]
@@ -925,7 +894,7 @@ class PyDirectory(Directory):
     """By default, all components in the Python installer
     can run from source."""
     def __init__(self, *args, **kw):
-        if not kw.has_key("componentflags"):
+        if "componentflags" not in kw:
             kw['componentflags'] = 2 #msidbComponentAttributesOptional
         Directory.__init__(self, *args, **kw)
 
@@ -959,7 +928,7 @@ def add_files(db):
     if not snapshot:
         # For releases, the Python DLL has the same version as the
         # installer package.
-        assert pyversion.split(".")[:3] == current_version.split("."), "%s != %s" % (pyversion, current_version)
+        assert pyversion.split(".")[:3] == current_version.split(".")
     dlldir.add_file("%s/python%s%s.dll" % (PCBUILD, major, minor),
                     version=pyversion,
                     language=installer.FileVersion(pydllsrc, 1))
@@ -988,10 +957,10 @@ def add_files(db):
     # Check if _ctypes.pyd exists
     have_ctypes = os.path.exists(srcdir+"/%s/_ctypes.pyd" % PCBUILD)
     if not have_ctypes:
-        print "WARNING: _ctypes.pyd not found, ctypes will not be included"
+        print("WARNING: _ctypes.pyd not found, ctypes will not be included")
         extensions.remove("_ctypes.pyd")
 
-    # Add all .py files in Lib, except lib-tk, test
+    # Add all .py files in Lib, except tkinter, test
     dirs={}
     pydirs = [(root,"Lib")]
     while pydirs:
@@ -1000,12 +969,12 @@ def add_files(db):
         parent, dir = pydirs.pop()
         if dir == ".svn" or dir.startswith("plat-"):
             continue
-        elif dir in ["lib-tk", "idlelib", "Icons"]:
+        elif dir in ["tkinter", "idlelib", "Icons"]:
             if not have_tcl:
                 continue
             tcltk.set_current()
         elif dir in ['test', 'tests', 'data', 'output']:
-            # test: Lib, Lib/email, Lib/bsddb, Lib/ctypes, Lib/sqlite3
+            # test: Lib, Lib/email, Lib/ctypes, Lib/sqlite3
             # tests: Lib/distutils
             # data: Lib/email/test
             # output: Lib/test
@@ -1018,8 +987,6 @@ def add_files(db):
         # Add additional files
         dirs[dir]=lib
         lib.glob("*.txt")
-        lib.glob("*.whl")
-        lib.glob("*.0")
         if dir=='site-packages':
             lib.add_file("README.txt", src="README")
             continue
@@ -1039,27 +1006,19 @@ def add_files(db):
             lib.add_file("audiotest.au")
             lib.add_file("cfgparser.1")
             lib.add_file("sgml_input.html")
+            lib.add_file("test.xml")
+            lib.add_file("test.xml.out")
             lib.add_file("testtar.tar")
             lib.add_file("test_difflib_expect.html")
             lib.add_file("check_soundcard.vbs")
             lib.add_file("empty.vbs")
             lib.add_file("Sine-1000Hz-300ms.aif")
-            lib.add_file("revocation.crl")
             lib.glob("*.uue")
             lib.glob("*.pem")
             lib.glob("*.pck")
             lib.add_file("zipdir.zip")
-        if dir=='tests' and parent.physical=='distutils':
-            lib.add_file("Setup.sample")
-        if dir=='audiodata':
-            lib.glob("*.*")
         if dir=='decimaltestdata':
             lib.glob("*.decTest")
-        if dir=='imghdrdata':
-            lib.glob("*.*")
-        if dir=='xmltestdata':
-            lib.glob("*.xml")
-            lib.add_file("test.xml.out")
         if dir=='output':
             lib.glob("test_*")
         if dir=='idlelib':
@@ -1067,7 +1026,6 @@ def add_files(db):
             lib.add_file("idle.bat")
         if dir=="Icons":
             lib.glob("*.gif")
-            lib.glob("*.ico")
             lib.add_file("idle.icns")
         if dir=="command" and parent.physical=="distutils":
             lib.glob("wininst*.exe")
@@ -1083,7 +1041,7 @@ def add_files(db):
                 if f.endswith(".au") or f.endswith(".gif"):
                     lib.add_file(f)
                 else:
-                    print "WARNING: New file %s in email/test/data" % f
+                    print("WARNING: New file %s in email/test/data" % f)
         for f in os.listdir(lib.absolute):
             if os.path.isdir(os.path.join(lib.absolute, f)):
                 pydirs.append((lib, f))
@@ -1098,7 +1056,7 @@ def add_files(db):
         if f=="_tkinter.pyd":
             continue
         if not os.path.exists(srcdir + "/" + PCBUILD + "/" + f):
-            print "WARNING: Missing extension", f
+            print("WARNING: Missing extension", f)
             continue
         dlls.append(f)
         lib.add_file(f)
@@ -1114,19 +1072,19 @@ def add_files(db):
     lib.add_file("sqlite3.dll")
     if have_tcl:
         if not os.path.exists("%s/%s/_tkinter.pyd" % (srcdir, PCBUILD)):
-            print "WARNING: Missing _tkinter.pyd"
+            print("WARNING: Missing _tkinter.pyd")
         else:
             lib.start_component("TkDLLs", tcltk)
             lib.add_file("_tkinter.pyd")
             dlls.append("_tkinter.pyd")
-            tcldir = os.path.normpath(srcdir+("/externals/tcltk%s/bin" % tclsuffix))
+            tcldir = os.path.normpath(srcdir+("/../tcltk%s/bin" % tclsuffix))
             for f in glob.glob1(tcldir, "*.dll"):
                 lib.add_file(f, src=os.path.join(tcldir, f))
     # check whether there are any unknown extensions
     for f in glob.glob1(srcdir+"/"+PCBUILD, "*.pyd"):
         if f.endswith("_d.pyd"): continue # debug version
         if f in dlls: continue
-        print "WARNING: Unknown extension", f
+        print("WARNING: Unknown extension", f)
 
     # Add headers
     default_feature.set_current()
@@ -1143,7 +1101,7 @@ def add_files(db):
         lib.add_file('libpython%s%s.a' % (major, minor))
     if have_tcl:
         # Add Tcl/Tk
-        tcldirs = [(root, 'externals/tcltk%s/lib' % tclsuffix, 'tcl')]
+        tcldirs = [(root, '../tcltk%s/lib' % tclsuffix, 'tcl')]
         tcltk.set_current()
         while tcldirs:
             parent, phys, dir = tcldirs.pop()
@@ -1202,10 +1160,6 @@ def add_registry(db):
                "InstallPath"),
               ("REGISTRY.doc", msilib.gen_uuid(), "TARGETDIR", registry_component, None,
                "Documentation"),
-              ("REGISTRY.path", msilib.gen_uuid(), "TARGETDIR", registry_component, None,
-               None),
-              ("REGISTRY.ensurepip", msilib.gen_uuid(), "TARGETDIR", registry_component, "EnsurePipRun",
-              None),
               ("REGISTRY.def", msilib.gen_uuid(), "TARGETDIR", registry_component,
                None, None)] + tcldata)
     # See "FeatureComponents Table".
@@ -1222,8 +1176,6 @@ def add_registry(db):
     add_data(db, "FeatureComponents",
              [(default_feature.id, "REGISTRY"),
               (htmlfiles.id, "REGISTRY.doc"),
-              (prepend_path.id, "REGISTRY.path"),
-              (pip_feature.id, "REGISTRY.ensurepip"),
               (ext_feature.id, "REGISTRY.def")] +
               tcldata
               )
@@ -1298,7 +1250,7 @@ def add_registry(db):
               ("InstallGroup", -1, prefix+r"\InstallPath\InstallGroup", "",
                "Python %s" % short_version, "REGISTRY"),
               ("PythonPath", -1, prefix+r"\PythonPath", "",
-               r"[TARGETDIR]Lib;[TARGETDIR]DLLs;[TARGETDIR]Lib\lib-tk", "REGISTRY"),
+               r"[TARGETDIR]Lib;[TARGETDIR]DLLs", "REGISTRY"),
               ("Documentation", -1, prefix+r"\Help\Main Python Documentation", "",
                "[TARGETDIR]Doc\\"+docfile , "REGISTRY.doc"),
               ("Modules", -1, prefix+r"\Modules", "+", None, "REGISTRY"),
@@ -1306,9 +1258,7 @@ def add_registry(db):
                "", r"[TARGETDIR]Python.exe", "REGISTRY.def"),
               ("DisplayIcon", -1,
                r"Software\Microsoft\Windows\CurrentVersion\Uninstall\%s" % product_code,
-               "DisplayIcon", "[TARGETDIR]python.exe", "REGISTRY"),
-              # Fake registry entry to allow installer to track whether ensurepip has been run
-              ("EnsurePipRun", -1, prefix+r"\EnsurePipRun", "", "#1", "REGISTRY.ensurepip"),
+               "DisplayIcon", "[TARGETDIR]python.exe", "REGISTRY")
               ])
     # Shortcuts, see "Shortcut Table"
     add_data(db, "Directory",
@@ -1343,17 +1293,7 @@ def add_registry(db):
               ])
     db.Commit()
 
-def build_pdbzip():
-    pdbexclude = ['kill_python.pdb', 'make_buildinfo.pdb',
-                  'make_versioninfo.pdb']
-    path = "python-%s%s-pdb.zip" % (full_current_version, msilib.arch_ext)
-    pdbzip = zipfile.ZipFile(path, 'w')
-    for f in glob.glob1(os.path.join(srcdir, PCBUILD), "*.pdb"):
-        if f not in pdbexclude and not f.endswith('_d.pdb'):
-            pdbzip.write(os.path.join(srcdir, PCBUILD, f), f)
-    pdbzip.close()
-
-db,msiname = build_database()
+db = build_database()
 try:
     add_features(db)
     add_ui(db)
@@ -1363,81 +1303,3 @@ try:
     db.Commit()
 finally:
     del db
-
-# Merge CRT into MSI file. This requires the database to be closed.
-mod_dir = os.path.join(os.environ["ProgramFiles"], "Common Files", "Merge Modules")
-if msilib.Win64:
-    modules = ["Microsoft_VC90_CRT_x86_x64.msm", "policy_9_0_Microsoft_VC90_CRT_x86_x64.msm"]
-else:
-    modules = ["Microsoft_VC90_CRT_x86.msm","policy_9_0_Microsoft_VC90_CRT_x86.msm"]
-
-for i, n in enumerate(modules):
-    modules[i] = os.path.join(mod_dir, n)
-
-def merge(msi, feature, rootdir, modules):
-    cab_and_filecount = []
-    # Step 1: Merge databases, extract cabfiles
-    m = msilib.MakeMerge2()
-    m.OpenLog("merge.log")
-    m.OpenDatabase(msi)
-    for module in modules:
-        print module
-        m.OpenModule(module,0)
-        m.Merge(feature, rootdir)
-        print "Errors:"
-        for e in m.Errors:
-            print e.Type, e.ModuleTable, e.DatabaseTable
-            print "   Modkeys:",
-            for s in e.ModuleKeys: print s,
-            print
-            print "   DBKeys:",
-            for s in e.DatabaseKeys: print s,
-            print
-        cabname = tempfile.mktemp(suffix=".cab")
-        m.ExtractCAB(cabname)
-        cab_and_filecount.append((cabname, len(m.ModuleFiles)))
-        m.CloseModule()
-    m.CloseDatabase(True)
-    m.CloseLog()
-
-    # Step 2: Add CAB files
-    i = msilib.MakeInstaller()
-    db = i.OpenDatabase(msi, constants.msiOpenDatabaseModeTransact)
-
-    v = db.OpenView("SELECT LastSequence FROM Media")
-    v.Execute(None)
-    maxmedia = -1
-    while 1:
-        r = v.Fetch()
-        if not r: break
-        seq = r.IntegerData(1)
-        if seq > maxmedia:
-            maxmedia = seq
-    print "Start of Media", maxmedia
-
-    for cabname, count in cab_and_filecount:
-        stream = "merged%d" % maxmedia
-        msilib.add_data(db, "Media",
-                [(maxmedia+1, maxmedia+count, None, "#"+stream, None, None)])
-        msilib.add_stream(db, stream,  cabname)
-        os.unlink(cabname)
-        maxmedia += count
-    # The merge module sets ALLUSERS to 1 in the property table.
-    # This is undesired; delete that
-    v = db.OpenView("DELETE FROM Property WHERE Property='ALLUSERS'")
-    v.Execute(None)
-    v.Close()
-    db.Commit()
-
-merge(msiname, "SharedCRT", "TARGETDIR", modules)
-
-# certname (from config.py) should be (a substring of)
-# the certificate subject, e.g. "Python Software Foundation"
-if certname:
-    os.system('signtool sign /n "%s" '
-      '/t http://timestamp.verisign.com/scripts/timestamp.dll '
-      '/d "Python %s" '
-      '%s' % (certname, full_current_version, msiname))
-
-if pdbzip:
-    build_pdbzip()
