@@ -7,13 +7,10 @@ their respective text boxes, click GO or hit return, and presto.
 """
 
 from Tkinter import *
-import Tkinter
-import string
 import websucker
-import sys
 import os
 import threading
-import Queue
+import queue
 import time
 
 VERBOSE = 2
@@ -142,7 +139,7 @@ class App:
 
     def go(self, event=None):
         if not self.msgq:
-            self.msgq = Queue.Queue(0)
+            self.msgq = queue.Queue(0)
             self.check_msgq()
         if not self.sucker:
             self.sucker = SuckerThread(self.msgq)
@@ -150,19 +147,19 @@ class App:
             return
         self.url_entry.selection_range(0, END)
         url = self.url_entry.get()
-        url = string.strip(url)
+        url = url.strip()
         if not url:
             self.top.bell()
             self.message("[Error: No URL entered]")
             return
         self.rooturl = url
-        dir = string.strip(self.dir_entry.get())
+        dir = self.dir_entry.get().strip()
         if not dir:
             self.sucker.savedir = None
         else:
             self.sucker.savedir = dir
             self.sucker.rootdir = os.path.dirname(
-                websucker.Sucker.savefilename(self, url))
+                websucker.Sucker.savefilename(self.sucker, url))
         self.go_button.configure(state=DISABLED)
         self.auto_button.configure(state=DISABLED)
         self.cancel_button.configure(state=NORMAL)
@@ -184,7 +181,7 @@ class App:
                 text = self.top.selection_get(selection=t)
             except TclError:
                 continue
-            text = string.strip(text)
+            text = text.strip()
             if text:
                 break
         if not text:
